"""
HTTP 服务器 - 对应 OpenClaw browser/server.js
"""

import asyncio
import os
import signal
from contextlib import asynccontextmanager

import uvicorn
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from .core.browser_registry import BrowserRegistry
from .models.config import BrowserConfig
from .routes.basic import set_browser_registry


class BrowserServer:
    """浏览器控制服务器 - 支持多浏览器实例

    生命周期管理：
    - idle_timeout: 空闲超时时间（秒），默认 3600 秒（1小时）
    - max_lifetime: 最大生命周期（秒），默认 3600 秒（1小时）
    """

    DEFAULT_LIFETIME = 3600  # 默认 1 小时

    def __init__(
        self,
        config: BrowserConfig | None = None,
        idle_timeout: float | None = None,
        max_lifetime: float | None = None,
        cleanup_interval: float = 60.0,
    ):
        self.config = config or BrowserConfig.from_env()

        # 从环境变量读取生命周期配置（优先级：参数 > 环境变量 > 默认值）
        env_idle_timeout = os.environ.get("QIANJI_IDLE_TIMEOUT")
        env_max_lifetime = os.environ.get("QIANJI_MAX_LIFETIME")

        final_idle_timeout = idle_timeout
        if final_idle_timeout is None and env_idle_timeout is not None:
            final_idle_timeout = float(env_idle_timeout)
        if final_idle_timeout is None:
            final_idle_timeout = self.DEFAULT_LIFETIME

        final_max_lifetime = max_lifetime
        if final_max_lifetime is None and env_max_lifetime is not None:
            final_max_lifetime = float(env_max_lifetime)
        if final_max_lifetime is None:
            final_max_lifetime = self.DEFAULT_LIFETIME

        self.browser_registry = BrowserRegistry(
            self.config,
            idle_timeout=final_idle_timeout,
            max_lifetime=final_max_lifetime,
            cleanup_interval=cleanup_interval,
        )
        self.app: FastAPI | None = None
        self.server: uvicorn.Server | None = None

    def create_app(self) -> FastAPI:
        """创建 FastAPI 应用"""

        @asynccontextmanager
        async def lifespan(app: FastAPI):
            """应用生命周期管理"""
            # 启动时初始化
            set_browser_registry(self.browser_registry, self.config)

            # 启动后台清理任务（如果配置了生命周期管理）
            if self.browser_registry.idle_timeout or self.browser_registry.max_lifetime:
                await self.browser_registry.start_cleanup_task()
                print(
                    f"✓ Browser lifecycle manager started (idle_timeout={self.browser_registry.idle_timeout}s, max_lifetime={self.browser_registry.max_lifetime}s)"
                )

            # 如果配置了自动启动，则启动默认浏览器
            if self.config.enabled and not self.config.attach_only:
                try:
                    browser_id = await self.browser_registry.create_browser(
                        self.config.default_profile
                    )
                    print(
                        f"✓ Browser auto-started: {browser_id} (profile: {self.config.default_profile})"
                    )
                except Exception as e:
                    print(f"⚠ Failed to auto-start browser: {e}")

            yield

            # 关闭时清理
            await self.browser_registry.stop_cleanup_task()
            await self.browser_registry.close_all()
            print("✓ All browsers stopped")

        app = FastAPI(
            title="qianji",
            description="Universal Browser Automation for AI - 100% OpenClaw Compatible",
            version="2.0.0",
            lifespan=lifespan,
        )

        # CORS 中间件
        app.add_middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )

        # 健康检查
        @app.get("/health")
        async def health_check():
            stats = self.browser_registry.get_stats()
            return {
                "status": "ok",
                "version": "2.0.0",
                "browsers": stats,
            }

        # 注册基础路由 (不带 /api/v1 前缀)
        from .routes import basic_router

        app.include_router(basic_router, tags=["basic"])

        # 注册 API 路由 (带 /api/v1 前缀)
        from .routes import (
            agent_act_router,
            agent_debug_router,
            agent_snapshot_router,
            agent_storage_router,
            tabs_router,
        )

        app.include_router(agent_snapshot_router, prefix="/api/v1", tags=["snapshot"])
        app.include_router(agent_act_router, prefix="/api/v1/act", tags=["act"])
        app.include_router(agent_storage_router, prefix="/api/v1/storage", tags=["storage"])
        app.include_router(agent_debug_router, prefix="/api/v1/debug", tags=["debug"])
        app.include_router(tabs_router, prefix="/api/v1/tabs", tags=["tabs"])

        self.app = app
        return app

    async def start(self, host: str = "0.0.0.0", port: int | None = None, log_level: str = "info"):
        """启动服务器"""
        port = port or self.config.control_port

        if self.app is None:
            self.create_app()

        config = uvicorn.Config(
            self.app,
            host=host,
            port=port,
            log_level=log_level,
            access_log=log_level != "critical",  # 静默模式下禁用访问日志
        )

        self.server = uvicorn.Server(config)

        # 设置信号处理
        for sig in (signal.SIGTERM, signal.SIGINT):
            asyncio.get_event_loop().add_signal_handler(sig, self._signal_handler)

        if log_level != "critical":
            print(f"🚀 qianji server starting on http://{host}:{port}")
        await self.server.serve()

    def _signal_handler(self):
        """信号处理"""
        if self.server:
            self.server.should_exit = True

    async def stop(self):
        """停止服务器"""
        if self.server:
            self.server.should_exit = True
            # 等待服务器停止
            while not self.server.started:
                await asyncio.sleep(0.1)


def main():
    """命令行入口"""
    import argparse

    parser = argparse.ArgumentParser(description="qianji - Universal Browser Automation")
    parser.add_argument("--host", default="0.0.0.0", help="Host to bind")
    parser.add_argument("--port", type=int, default=None, help="Port to bind")
    parser.add_argument("--headless", action="store_true", help="Run browser in headless mode")
    parser.add_argument("--no-sandbox", action="store_true", help="Disable sandbox")
    parser.add_argument("--browser-path", help="Path to browser executable")
    parser.add_argument("--quiet", action="store_true", help="Quiet mode (minimal output)")
    # 生命周期管理参数（默认 1 小时 = 3600 秒）
    parser.add_argument(
        "--idle-timeout",
        type=float,
        default=None,
        help="Browser idle timeout in seconds (default: 3600 = 1 hour, 0 to disable)",
    )
    parser.add_argument(
        "--max-lifetime",
        type=float,
        default=None,
        help="Browser maximum lifetime in seconds (default: 3600 = 1 hour, 0 to disable)",
    )
    parser.add_argument(
        "--cleanup-interval",
        type=float,
        default=60.0,
        help="Cleanup check interval in seconds (default: 60)",
    )

    args = parser.parse_args()

    # 从环境变量或参数加载配置
    config = BrowserConfig.from_env()

    if args.headless:
        config.headless = True
    if args.no_sandbox:
        config.no_sandbox = True
    if args.browser_path:
        config.executable_path = args.browser_path

    # 静默模式
    log_level = "critical" if args.quiet else "info"

    # 创建并启动服务器（带生命周期管理）
    server = BrowserServer(
        config,
        idle_timeout=args.idle_timeout,
        max_lifetime=args.max_lifetime,
        cleanup_interval=args.cleanup_interval,
    )

    try:
        asyncio.run(server.start(host=args.host, port=args.port, log_level=log_level))
    except KeyboardInterrupt:
        if not args.quiet:
            print("\n👋 Goodbye!")


if __name__ == "__main__":
    main()



"""
浏览器注册表 - 管理多个浏览器实例
支持 N 浏览器 × M 页面架构
"""

import asyncio
import os
from dataclasses import dataclass, field
from typing import Any

from ..models.config import BrowserConfig
from .browser_manager import BrowserManager


@dataclass
class BrowserInstance:
    """浏览器实例包装器"""

    browser_id: str
    manager: BrowserManager
    profile_name: str
    user_data_dir: str
    created_at: float = field(default_factory=lambda: asyncio.get_event_loop().time())
    last_activity_at: float = field(default_factory=lambda: asyncio.get_event_loop().time())
    # 实例级别的生命周期配置（覆盖全局配置）
    idle_timeout: float | None = None
    max_lifetime: float | None = None


class BrowserRegistry:
    """浏览器注册表 - 管理多个浏览器实例

    支持生命周期管理：
    - idle_timeout: 空闲超时时间（秒），超过此时间未活动的浏览器将被自动关闭
    - max_lifetime: 最大生命周期（秒），超过此时间的浏览器将被强制关闭
    """

    def __init__(
        self,
        config: BrowserConfig,
        idle_timeout: float | None = None,
        max_lifetime: float | None = None,
        cleanup_interval: float = 60.0,
    ):
        self.config = config
        self._browsers: dict[str, BrowserInstance] = {}
        self._counter = 0
        self._lock = asyncio.Lock()

        # 生命周期配置
        self.idle_timeout = idle_timeout  # 空闲超时（秒）
        self.max_lifetime = max_lifetime  # 最大生命周期（秒）
        self.cleanup_interval = cleanup_interval  # 清理检查间隔（秒）

        # 后台清理任务
        self._cleanup_task: asyncio.Task | None = None
        self._running = False

    def _generate_browser_id(self) -> str:
        """生成唯一的浏览器ID"""
        self._counter += 1
        return f"browser_{self._counter}"

    def _get_user_data_dir(self, browser_id: str, profile_name: str) -> str:
        """为每个浏览器实例生成独立的用户数据目录"""
        base_dir = os.path.expanduser("~/.qianji/user_data")
        # 使用 browser_id 作为目录名，确保每个实例独立
        user_data_dir = os.path.join(base_dir, f"{profile_name}_{browser_id}")
        os.makedirs(user_data_dir, exist_ok=True)
        return user_data_dir

    async def create_browser(
        self,
        profile_name: str | None = None,
        browser_id: str | None = None,
        idle_timeout: float | None = None,
        max_lifetime: float | None = None,
        headless: bool | None = None,
        no_sandbox: bool | None = None,
        args: list[str] | None = None,
    ) -> str:
        """
        创建新浏览器实例

        Args:
            profile_name: 配置文件名称
            browser_id: 指定的浏览器ID（可选，自动生成）
            idle_timeout: 空闲超时时间（秒），默认使用全局配置
            max_lifetime: 最大生命周期（秒），默认使用全局配置
            headless: 是否使用无头模式，默认使用全局配置
            no_sandbox: 是否禁用沙箱，默认使用全局配置
            args: 额外的浏览器启动参数

        Returns:
            browser_id: 浏览器实例ID
        """
        async with self._lock:
            # 生成或验证 browser_id
            if browser_id is None:
                browser_id = self._generate_browser_id()
            elif browser_id in self._browsers:
                raise ValueError(f"Browser ID already exists: {browser_id}")

            profile = profile_name or self.config.default_profile

            # 为每个实例创建独立的用户数据目录
            user_data_dir = self._get_user_data_dir(browser_id, profile)

            # 创建临时配置（如果需要覆盖 headless/no_sandbox/args）
            import copy

            config = copy.deepcopy(self.config)
            if headless is not None:
                config.headless = headless
            if no_sandbox is not None:
                config.no_sandbox = no_sandbox
            if args:
                # 将额外参数添加到所有配置文件
                for profile_config in config.profiles.values():
                    profile_config.args.extend(args)

            # 创建新的 BrowserManager，传入独立的用户数据目录
            manager = BrowserManager(config)

            # 启动浏览器（使用独立的用户数据目录）
            await manager.start(profile, user_data_dir=user_data_dir)

            # 使用实例配置或回退到全局配置
            instance_idle_timeout = idle_timeout if idle_timeout is not None else self.idle_timeout
            instance_max_lifetime = max_lifetime if max_lifetime is not None else self.max_lifetime

            # 注册实例
            instance = BrowserInstance(
                browser_id=browser_id,
                manager=manager,
                profile_name=profile,
                user_data_dir=user_data_dir,
                idle_timeout=instance_idle_timeout,
                max_lifetime=instance_max_lifetime,
            )
            self._browsers[browser_id] = instance

            return browser_id

    async def get_browser(self, browser_id: str) -> BrowserManager | None:
        """获取浏览器管理器"""
        instance = self._browsers.get(browser_id)
        if instance and instance.manager.is_running:
            # 更新最后活动时间
            instance.last_activity_at = asyncio.get_event_loop().time()
            return instance.manager
        return None

    def update_activity(self, browser_id: str) -> bool:
        """更新浏览器最后活动时间

        Args:
            browser_id: 浏览器ID

        Returns:
            bool: 是否成功更新
        """
        instance = self._browsers.get(browser_id)
        if instance:
            instance.last_activity_at = asyncio.get_event_loop().time()
            return True
        return False

    def get_browser_age(self, browser_id: str) -> float | None:
        """获取浏览器已运行时间（秒）

        Args:
            browser_id: 浏览器ID

        Returns:
            float: 运行时间（秒），如果不存在返回 None
        """
        instance = self._browsers.get(browser_id)
        if instance:
            return asyncio.get_event_loop().time() - instance.created_at
        return None

    def get_browser_idle_time(self, browser_id: str) -> float | None:
        """获取浏览器空闲时间（秒）

        Args:
            browser_id: 浏览器ID

        Returns:
            float: 空闲时间（秒），如果不存在返回 None
        """
        instance = self._browsers.get(browser_id)
        if instance:
            return asyncio.get_event_loop().time() - instance.last_activity_at
        return None

    async def start_cleanup_task(self):
        """启动后台清理任务"""
        if self._cleanup_task is not None:
            return

        self._running = True
        self._cleanup_task = asyncio.create_task(self._cleanup_loop())

    async def stop_cleanup_task(self):
        """停止后台清理任务"""
        self._running = False
        if self._cleanup_task:
            self._cleanup_task.cancel()
            try:
                await self._cleanup_task
            except asyncio.CancelledError:
                pass
            self._cleanup_task = None

    async def _cleanup_loop(self):
        """后台清理循环"""
        while self._running:
            try:
                await asyncio.sleep(self.cleanup_interval)
                await self._cleanup_expired_browsers()
            except asyncio.CancelledError:
                break
            except Exception:
                # 清理出错不应中断循环
                pass

    async def _cleanup_expired_browsers(self):
        """清理过期浏览器（支持实例级别配置）"""
        now = asyncio.get_event_loop().time()
        to_close = []

        async with self._lock:
            for browser_id, instance in list(self._browsers.items()):
                if not instance.manager.is_running:
                    continue

                # 使用实例级别的 max_lifetime，如果没有则使用全局配置
                max_lifetime = (
                    instance.max_lifetime
                    if instance.max_lifetime is not None
                    else self.max_lifetime
                )
                if max_lifetime is not None:
                    age = now - instance.created_at
                    if age >= max_lifetime:
                        to_close.append((browser_id, "max_lifetime"))
                        continue

                # 使用实例级别的 idle_timeout，如果没有则使用全局配置
                idle_timeout = (
                    instance.idle_timeout
                    if instance.idle_timeout is not None
                    else self.idle_timeout
                )
                if idle_timeout is not None:
                    idle_time = now - instance.last_activity_at
                    if idle_time >= idle_timeout:
                        to_close.append((browser_id, "idle_timeout"))

        # 在锁外关闭浏览器
        for browser_id, reason in to_close:
            try:
                await self.close_browser(browser_id)
            except Exception:
                pass

    async def ensure_browser(
        self, browser_id: str | None = None, profile_name: str | None = None
    ) -> tuple[str, BrowserManager]:
        """
        确保浏览器实例存在并运行

        Args:
            browser_id: 浏览器ID（可选）
            profile_name: 配置文件名称（创建新实例时使用）

        Returns:
            (browser_id, manager): 浏览器ID和管理器
        """
        async with self._lock:
            # 如果指定了 browser_id，尝试获取
            if browser_id:
                instance = self._browsers.get(browser_id)
                if instance:
                    if not instance.manager.is_running:
                        # 重新启动
                        await instance.manager.start()
                    # 更新活动时间
                    instance.last_activity_at = asyncio.get_event_loop().time()
                    return browser_id, instance.manager

            # 检查是否有任何运行的浏览器
            for bid, instance in self._browsers.items():
                if instance.manager.is_running:
                    # 更新活动时间
                    instance.last_activity_at = asyncio.get_event_loop().time()
                    return bid, instance.manager

            # 创建新浏览器
            new_browser_id = await self.create_browser(profile_name)
            return new_browser_id, self._browsers[new_browser_id].manager

    async def close_browser(self, browser_id: str) -> bool:
        """关闭指定浏览器"""
        async with self._lock:
            instance = self._browsers.get(browser_id)
            if not instance:
                return False

            await instance.manager.stop()
            del self._browsers[browser_id]
            return True

    async def close_all(self):
        """关闭所有浏览器"""
        async with self._lock:
            for instance in list(self._browsers.values()):
                await instance.manager.stop()
            self._browsers.clear()

    def list_browsers(self) -> list[dict[str, Any]]:
        """列出所有浏览器实例，包含生命周期信息"""
        now = asyncio.get_event_loop().time()
        return [
            {
                "browserId": bid,
                "profileName": instance.profile_name,
                "running": instance.manager.is_running,
                "tabs": len(instance.manager.tab_manager.tabs),
                "userDataDir": instance.manager._user_data_dir,
                # 生命周期信息
                "createdAt": instance.created_at,
                "lastActivityAt": instance.last_activity_at,
                "age": now - instance.created_at,
                "idleTime": now - instance.last_activity_at,
                # 实例级别的生命周期配置
                "lifecycle": {
                    "idleTimeout": instance.idle_timeout,
                    "maxLifetime": instance.max_lifetime,
                },
            }
            for bid, instance in self._browsers.items()
        ]

    def get_stats(self) -> dict[str, Any]:
        """获取统计信息，包含生命周期配置"""
        total_browsers = len(self._browsers)
        running_browsers = sum(1 for b in self._browsers.values() if b.manager.is_running)
        total_tabs = sum(len(b.manager.tab_manager.tabs) for b in self._browsers.values())

        return {
            "totalBrowsers": total_browsers,
            "runningBrowsers": running_browsers,
            "totalTabs": total_tabs,
            # 生命周期配置
            "lifecycle": {
                "idleTimeout": self.idle_timeout,
                "maxLifetime": self.max_lifetime,
                "cleanupInterval": self.cleanup_interval,
            },
            "browsers": self.list_browsers(),
        }



"""
MCP 包装脚本 - 简化版
- 确保全局只有一个 server 在 18796 端口
- 复用已有的 qianji-server 和 qianji-mcp 命令
"""

import asyncio
import os
import subprocess
import sys
import time
from pathlib import Path

DEFAULT_PORT = 18796


def log(message: str):
    """输出日志到 stderr"""
    print(message, file=sys.stderr, flush=True)


def is_server_running(port: int) -> bool:
    """检查 server 是否正在运行"""
    try:
        import http.client

        conn = http.client.HTTPConnection("127.0.0.1", port, timeout=2)
        conn.request("GET", "/health")
        response = conn.getresponse()
        return response.status == 200
    except:
        return False


def wait_for_server(port: int, timeout: float = 60) -> bool:
    """等待 server 启动"""
    start = time.time()
    while time.time() - start < timeout:
        if is_server_running(port):
            return True
        time.sleep(0.1)
    return False


def start_server(port: int, quiet: bool = True) -> subprocess.Popen:
    """启动 qianji-server"""
    env = os.environ.copy()
    env["QIANJI_CONTROL_PORT"] = str(port)

    # 确保 PYTHONPATH 包含项目根目录
    project_root = str(Path(__file__).parent.parent)
    current_pythonpath = env.get("PYTHONPATH", "")
    if project_root not in current_pythonpath:
        env["PYTHONPATH"] = (
            f"{project_root}:{current_pythonpath}" if current_pythonpath else project_root
        )

    # 使用绝对路径的 Python 解释器来运行 server
    python = sys.executable
    cmd = [python, "-m", "qianji.server", "--port", str(port)]
    if quiet:
        cmd.append("--quiet")

    if not quiet:
        log(f"  Command: {' '.join(cmd)}")
        log(f"  PYTHONPATH: {env.get('PYTHONPATH', 'not set')}")

    # 先检查是否已有 server 在运行（避免重复启动）
    if is_server_running(port):
        if not quiet:
            log(f"  Server already running on port {port}")
        return None

    # 启动 server，完全脱离当前进程
    process = subprocess.Popen(
        cmd,
        env=env,
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL if quiet else None,
        stdin=subprocess.DEVNULL,
        start_new_session=True,
    )

    if not quiet:
        log(f"  Started process with PID: {process.pid}")

    # 等待 server 就绪
    if wait_for_server(port, timeout=60):
        if not quiet:
            log(f"  Server is ready on port {port}")
        return process

    # 启动失败，检查进程状态
    exit_code = process.poll()
    if not quiet:
        log(f"  Process exit code: {exit_code}")

    # 终止进程
    try:
        process.terminate()
        process.wait(timeout=2)
    except:
        process.kill()

    # 再次检查是否已经有其他 server 在运行（可能是其他进程启动的）
    if is_server_running(port):
        if not quiet:
            log(f"  Another server is now running on port {port}")
        return None

    raise RuntimeError(f"Server failed to start on port {port} (exit code: {exit_code})")


def ensure_server(port: int = DEFAULT_PORT, quiet: bool = True) -> bool:
    """确保 server 在运行"""
    if is_server_running(port):
        if not quiet:
            log(f"✓ Server already running on port {port}")
        return True

    if not quiet:
        log(f"Starting server on port {port}...")

    log("DEBUG: About to start server...")  # 调试信息
    process = start_server(port, quiet=quiet)
    log(f"DEBUG: start_server returned: {process}")  # 调试信息

    if process is None:
        log("DEBUG: Server already running or started by another process")
        return True

    if not quiet:
        log(f"✓ Server started (PID: {process.pid})")

    log("DEBUG: ensure_server completed successfully")
    return True


def run_mcp(port: int, quiet: bool = True):
    """运行 MCP server"""
    from qianji.mcp import QianjiMCPServer

    if not quiet:
        log(f"Starting MCP server on port {port}...")

    # 检查 stdin 是否可用
    if sys.stdin.isatty():
        log("WARNING: stdin is a tty, MCP may not work correctly")
        log("MCP expects JSON-RPC messages from stdin")

    mcp_server = QianjiMCPServer(base_url=f"http://127.0.0.1:{port}")
    asyncio.run(mcp_server.run())


def main():
    import argparse

    parser = argparse.ArgumentParser(description="qianji MCP Wrapper")
    parser.add_argument("--port", type=int, default=DEFAULT_PORT)
    parser.add_argument("--mcp", action="store_true", help="Run MCP server")
    parser.add_argument("--verbose", action="store_true")

    args = parser.parse_args()
    quiet = not args.verbose

    try:
        # 确保 server 运行
        ensure_server(args.port, quiet=quiet)

        # 运行 MCP
        if args.mcp:
            run_mcp(args.port, quiet=quiet)

    except Exception as e:
        log(f"✗ Error: {e}")
        import traceback

        traceback.print_exc(file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()



import asyncio
import os
import sys

from playwright.async_api import async_playwright

from qianji.core.pw_client import PlaywrightClient

# 获取当前文件目录下的 fixture 路径
FIXTURE_DIR = os.path.join(os.path.dirname(__file__), "..", "fixtures")
TEST_HTML = f"file://{os.path.abspath(os.path.join(FIXTURE_DIR, 'v3_test.html'))}"


async def test_full_v3_capabilities():
    print("Starting V3 Integration Test...")
    async with async_playwright() as p:
        print("Launching browser with --no-sandbox...")
        browser = await p.chromium.launch(
            headless=True, args=["--no-sandbox", "--disable-setuid-sandbox"]
        )
        context = await browser.new_context()
        page = await context.new_page()

        try:
            client = PlaywrightClient()

            # 1. 导航与快照 (Cap 1, 4, 5)
            print(f"Navigating to {TEST_HTML}...")
            await client.navigate(page, TEST_HTML)
            print("Creating snapshot...")
            snapshot = await client.create_snapshot(page, max_elements=100)

            # 验证 iframe 提取 (Cap 1)
            print("Verifying iframe extraction...")
            iframe_elements = [e for e in snapshot.elements.values() if e.frame_id]
            print(f"Found {len(iframe_elements)} elements in iframe")
            assert len(iframe_elements) > 0, "Should extract elements from iframe"

            iframe_btn = next((e for e in iframe_elements if e.name == "Iframe Button"), None)
            assert iframe_btn is not None
            assert iframe_btn.frame_id == "f1"
            print("✓ Iframe extraction verified")

            # 验证裁剪 (Cap 5)
            print(f"Verifying truncation: {len(snapshot.elements)}/{snapshot.total_elements_found}")
            assert len(snapshot.elements) <= 100
            assert snapshot.total_elements_found > 200
            print("✓ Truncation verified")

            # 验证 DOM Hash (Cap 4)
            print("Verifying DOM Hash...")
            old_hash = snapshot.dom_hash
            assert old_hash != ""
            await page.click("#change-dom-btn")
            await asyncio.sleep(0.5)
            current_hash = await page.evaluate(
                "() => `${location.href}|${document.querySelectorAll('a,button,input,select,textarea,[role]').length}|${document.body.innerText.length}`"
            )
            assert old_hash != current_hash
            print("✓ DOM Hash change detected")

            # 2. 弹窗处理 (Cap 2)
            print("Verifying Dialog handling...")
            client.set_dialog_mode("accept")
            alert_ref = next(e.ref for e in snapshot.elements.values() if e.name == "Trigger Alert")
            await client.click_by_ref(page, alert_ref, snapshot)

            history = client.get_dialog_history()
            assert len(history) > 0
            assert history[-1]["message"] == "Hello from Alert"
            print(f"✓ Dialog verified: {history[-1]['message']}")

            # 3. Iframe 交互 (Cap 1)
            print("Verifying Iframe interaction...")
            iframe_btn_ref = next(
                e.ref for e in snapshot.elements.values() if e.name == "Iframe Button"
            )
            await client.click_by_ref(page, iframe_btn_ref, snapshot)

            content = await client.get_text_by_ref(page, iframe_btn_ref, snapshot)
            assert content == "Iframe Clicked"
            print("✓ Iframe interaction verified")

            # 4. 标注截图 (Cap 6)
            print("Verifying Annotated Screenshot...")
            temp_img = "/tmp/annotated_test.png"
            await client.annotated_screenshot(page, temp_img, snapshot)
            assert os.path.exists(temp_img)
            assert os.path.getsize(temp_img) > 0
            print(f"✓ Annotated screenshot saved to {temp_img}")

            print("\nALL V3 INTEGRATION TESTS PASSED! ✅")

        except Exception as e:
            print(f"\nTEST FAILED: {e}")
            import traceback

            traceback.print_exc()
            sys.exit(1)
        finally:
            await browser.close()


if __name__ == "__main__":
    asyncio.run(test_full_v3_capabilities())



"""
调试路由 - 增强版
控制台日志、错误、网络请求
使用 deque 限制存储大小
"""

import logging
from collections import deque

from fastapi import APIRouter
from pydantic import BaseModel

from ..models.responses import GenericResponse

logger = logging.getLogger("qianji.routes.debug")
router = APIRouter()

# 全局日志存储 — 使用 deque 限制大小
MAX_LOG_SIZE = 500

_console_logs: dict[str, deque] = {}
_page_errors: dict[str, deque] = {}
_network_requests: dict[str, deque] = {}


def _get_log_key(browser_id: str, target_id: str) -> str:
    return f"{browser_id}:{target_id}"


def _get_logs(store: dict[str, deque], key: str) -> deque:
    if key not in store:
        store[key] = deque(maxlen=MAX_LOG_SIZE)
    return store[key]


def cleanup_logs(browser_id: str, target_id: str):
    """清理指定浏览器/标签页的日志"""
    key = _get_log_key(browser_id, target_id)
    _console_logs.pop(key, None)
    _page_errors.pop(key, None)
    _network_requests.pop(key, None)


def cleanup_browser_logs(browser_id: str):
    """清理指定浏览器的所有日志"""
    keys_to_remove = [k for k in _console_logs if k.startswith(f"{browser_id}:")]
    for key in keys_to_remove:
        _console_logs.pop(key, None)
        _page_errors.pop(key, None)
        _network_requests.pop(key, None)


class LogEntry(BaseModel):
    browserId: str | None = None
    targetId: str | None = None
    type: str = "log"
    text: str


class GetLogsRequest(BaseModel):
    browserId: str | None = None
    targetId: str | None = None
    limit: int = 50


class ClearLogsRequest(BaseModel):
    browserId: str | None = None
    targetId: str | None = None


@router.post("/console/log")
async def add_console_log(entry: LogEntry):
    """添加控制台日志"""
    browser_id = entry.browserId or "default"
    target_id = entry.targetId or "default"
    key = _get_log_key(browser_id, target_id)

    logs = _get_logs(_console_logs, key)
    logs.append(
        {
            "type": entry.type,
            "text": entry.text[:2000],  # 限制单条日志长度
        }
    )

    return GenericResponse(ok=True, success=True)


@router.post("/console/get", response_model=GenericResponse)
async def get_console_logs(request: GetLogsRequest):
    """获取控制台日志"""
    browser_id = request.browserId or "default"
    target_id = request.targetId or "default"
    key = _get_log_key(browser_id, target_id)

    logs = list(_get_logs(_console_logs, key))
    if request.limit:
        logs = logs[-request.limit :]

    return GenericResponse(
        ok=True,
        success=True,
        data={"logs": logs, "count": len(logs)},
    )


@router.post("/console/clear")
async def clear_console_logs(request: ClearLogsRequest):
    """清除控制台日志"""
    browser_id = request.browserId or "default"
    target_id = request.targetId or "default"
    cleanup_logs(browser_id, target_id)

    return GenericResponse(ok=True, success=True)


@router.post("/errors/get", response_model=GenericResponse)
async def get_page_errors(request: GetLogsRequest):
    """获取页面错误"""
    browser_id = request.browserId or "default"
    target_id = request.targetId or "default"
    key = _get_log_key(browser_id, target_id)

    errors = list(_get_logs(_page_errors, key))
    if request.limit:
        errors = errors[-request.limit :]

    return GenericResponse(
        ok=True,
        success=True,
        data={"errors": errors, "count": len(errors)},
    )


@router.post("/errors/clear")
async def clear_page_errors(request: ClearLogsRequest):
    """清除页面错误"""
    browser_id = request.browserId or "default"
    target_id = request.targetId or "default"
    cleanup_logs(browser_id, target_id)

    return GenericResponse(ok=True, success=True)


@router.post("/network/get", response_model=GenericResponse)
async def get_network_requests(request: GetLogsRequest):
    """获取网络请求"""
    browser_id = request.browserId or "default"
    target_id = request.targetId or "default"
    key = _get_log_key(browser_id, target_id)

    requests_log = list(_get_logs(_network_requests, key))
    if request.limit:
        requests_log = requests_log[-request.limit :]

    return GenericResponse(
        ok=True,
        success=True,
        data={"requests": requests_log, "count": len(requests_log)},
    )


@router.post("/network/clear")
async def clear_network_requests(request: ClearLogsRequest):
    """清除网络请求"""
    browser_id = request.browserId or "default"
    target_id = request.targetId or "default"
    cleanup_logs(browser_id, target_id)

    return GenericResponse(ok=True, success=True)


# ==== 日志注入 API（供 BrowserManager 使用）====


def inject_console_log(browser_id: str, target_id: str, log_type: str, text: str):
    """注入控制台日志（供其他模块调用）"""
    key = _get_log_key(browser_id, target_id)
    logs = _get_logs(_console_logs, key)
    logs.append({"type": log_type, "text": text[:2000]})


def inject_page_error(browser_id: str, target_id: str, error: str):
    """注入页面错误"""
    key = _get_log_key(browser_id, target_id)
    errors = _get_logs(_page_errors, key)
    errors.append({"error": error[:2000]})


def inject_network_request(browser_id: str, target_id: str, url: str, method: str, status: int):
    """注入网络请求（只记录关键信息，不记录 headers/body）"""
    key = _get_log_key(browser_id, target_id)
    requests_log = _get_logs(_network_requests, key)
    requests_log.append(
        {
            "url": url[:500],
            "method": method,
            "status": status,
        }
    )



"""
MCP (Model Context Protocol) 支持 - 增强版
让 qianji 可以被 Claude Desktop, Cursor 等工具使用
优化工具设计: 复合工具、精简返回、引导性描述
"""

import asyncio
import json
import logging
import sys
from typing import Any

import httpx

logger = logging.getLogger("qianji.mcp")


class QianjiMCPServer:
    """MCP 服务器实现 - 增强版"""

    def __init__(self, base_url: str = "http://localhost:18796"):
        self.base_url = base_url
        self.client = httpx.AsyncClient(timeout=60.0)
        self.tools = self._define_tools()

    def _define_tools(self) -> list[dict[str, Any]]:
        """定义 MCP 工具列表 - 增强版，带引导性描述和复合工具"""
        return [
            # ============ 导航 ============
            {
                "name": "browser_navigate",
                "description": (
                    "Navigate to a URL. After navigation, call browser_snapshot to see "
                    "the page elements. Or use browser_navigate_and_snapshot to do both in one step."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "url": {"type": "string", "description": "URL to navigate to"},
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                        "waitUntil": {
                            "type": "string",
                            "description": "Wait strategy: 'domcontentloaded' (fast, default), 'load', 'networkidle' (slow)",
                            "default": "domcontentloaded",
                        },
                    },
                    "required": ["url"],
                },
            },
            # ============ 复合: 导航+快照 ============
            {
                "name": "browser_navigate_and_snapshot",
                "description": (
                    "Navigate to a URL AND immediately return the page snapshot with interactive elements. "
                    "This is the preferred way to navigate — saves one round trip compared to calling "
                    "browser_navigate then browser_snapshot separately."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "url": {"type": "string", "description": "URL to navigate to"},
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                        "waitUntil": {
                            "type": "string",
                            "description": "Wait strategy: 'domcontentloaded' (fast, default), 'load', 'networkidle' (slow)",
                            "default": "domcontentloaded",
                        },
                    },
                    "required": ["url"],
                },
            },
            # ============ 快照 ============
            {
                "name": "browser_snapshot",
                "description": (
                    "Get the current page state with all interactive elements. Returns [ref] labels "
                    "like [e1], [e2] for each element. Use these refs with browser_click, browser_type, etc. "
                    "IMPORTANT: Always call this after page navigation or state changes to get fresh refs. "
                    "Refs from previous snapshots become invalid after page changes."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                        "maxElements": {
                            "type": "integer",
                            "description": "Max interactive elements to return (default: 150)",
                            "default": 150,
                        },
                        "viewportOnly": {
                            "type": "boolean",
                            "description": "Only return elements in current viewport",
                            "default": False,
                        },
                    },
                },
            },
            # ============ 点击 ============
            {
                "name": "browser_click",
                "description": (
                    "Click an element by its ref (e.g. 'e1'). Returns the updated page snapshot "
                    "automatically — no need to call browser_snapshot afterwards unless the page "
                    "navigated to a completely new URL."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "ref": {
                            "type": "string",
                            "description": "Element ref from snapshot (e.g. 'e1')",
                        },
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                        "doubleClick": {
                            "type": "boolean",
                            "description": "Double-click instead of single click",
                            "default": False,
                        },
                    },
                    "required": ["ref"],
                },
            },
            # ============ 输入 ============
            {
                "name": "browser_type",
                "description": (
                    "Type text into an input field by its ref. Simulates keyboard typing. "
                    "Use 'submit: true' to press Enter after typing (e.g. for search boxes). "
                    "For clearing and replacing text, use browser_fill instead. "
                    "Returns updated snapshot automatically."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "ref": {"type": "string", "description": "Element ref from snapshot"},
                        "text": {"type": "string", "description": "Text to type"},
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                        "submit": {
                            "type": "boolean",
                            "description": "Press Enter after typing",
                            "default": False,
                        },
                    },
                    "required": ["ref", "text"],
                },
            },
            # ============ 填充 ============
            {
                "name": "browser_fill",
                "description": (
                    "Fill (clear and set) an input field by its ref. Unlike browser_type, this "
                    "clears existing text first. Best for form fields. Returns updated snapshot."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "ref": {"type": "string", "description": "Element ref from snapshot"},
                        "text": {"type": "string", "description": "Text to fill"},
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["ref", "text"],
                },
            },
            # ============ 文件上传 ============
            {
                "name": "browser_upload_file",
                "description": (
                    "Upload file(s) to a file input or by clicking an element that triggers "
                    "a file chooser. Returns updated snapshot."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "ref": {
                            "type": "string",
                            "description": "Element ref (input[type=file] or button)",
                        },
                        "filePaths": {
                            "type": "array",
                            "items": {"type": "string"},
                            "description": "Absolute paths of files to upload",
                        },
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["ref", "filePaths"],
                },
            },
            # ============ 弹窗管理 ============
            {
                "name": "browser_handle_dialog",
                "description": (
                    "Set how to handle future dialogs (alert/confirm/prompt). "
                    "Default is 'accept'."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "mode": {
                            "type": "string",
                            "description": "Mode: 'accept' (default), 'dismiss', 'custom'",
                            "enum": ["accept", "dismiss", "custom"],
                        },
                        "customText": {
                            "type": "string",
                            "description": "Text for prompts in 'custom' mode",
                        },
                    },
                    "required": ["mode"],
                },
            },
            {
                "name": "browser_dialog_history",
                "description": "Get history of recent dialogs (alerts, confirms, prompts encountered).",
                "inputSchema": {
                    "type": "object",
                    "properties": {},
                },
            },
            # ============ 下载查询 ============
            {
                "name": "browser_get_downloads",
                "description": "Get list of files downloaded during this session.",
                "inputSchema": {
                    "type": "object",
                    "properties": {},
                },
            },
            # ============ 滚动 ============
            {
                "name": "browser_scroll",
                "description": (
                    "Scroll the page. Use direction ('up'/'down'/'left'/'right') and amount in pixels. "
                    "Or provide a ref to scroll a specific element into view."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "direction": {
                            "type": "string",
                            "description": "Scroll direction",
                            "enum": ["up", "down", "left", "right"],
                            "default": "down",
                        },
                        "amount": {
                            "type": "integer",
                            "description": "Pixels to scroll",
                            "default": 500,
                        },
                        "ref": {
                            "type": "string",
                            "description": "Scroll element with this ref into view",
                        },
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                },
            },
            # ============ 悬停 ============
            {
                "name": "browser_hover",
                "description": "Hover over an element by its ref. Useful for triggering tooltips or dropdown menus.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "ref": {"type": "string", "description": "Element ref from snapshot"},
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["ref"],
                },
            },
            # ============ 选择 ============
            {
                "name": "browser_select",
                "description": (
                    "Select option(s) from a dropdown/select element. "
                    "Pass values as an array of option values. Returns updated snapshot."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "ref": {
                            "type": "string",
                            "description": "Element ref of the select/combobox",
                        },
                        "values": {
                            "type": "array",
                            "items": {"type": "string"},
                            "description": "Option value(s) to select",
                        },
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["ref", "values"],
                },
            },
            # ============ 按键 ============
            {
                "name": "browser_press",
                "description": (
                    "Press a keyboard key. Examples: 'Enter', 'Tab', 'Escape', 'ArrowDown', "
                    "'Control+a', 'Meta+c'. Use after focusing an element."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "key": {
                            "type": "string",
                            "description": "Key to press (e.g. 'Enter', 'Tab', 'Escape')",
                        },
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["key"],
                },
            },
            # ============ 获取文本 ============
            {
                "name": "browser_get_text",
                "description": "Get the text content of a specific element by its ref.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "ref": {"type": "string", "description": "Element ref from snapshot"},
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["ref"],
                },
            },
            # ============ 获取属性 ============
            {
                "name": "browser_get_attribute",
                "description": "Get an attribute value of an element (e.g. 'href', 'src', 'class').",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "ref": {"type": "string", "description": "Element ref from snapshot"},
                        "attribute": {
                            "type": "string",
                            "description": "Attribute name (e.g. 'href', 'src')",
                        },
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["ref", "attribute"],
                },
            },
            # ============ 等待元素 ============
            {
                "name": "browser_wait_for_element",
                "description": (
                    "Wait for an element to appear on the page. "
                    "Use CSS selectors like '#myId', '.myClass', 'button[type=submit]'. "
                    "Useful when waiting for dynamic content to load."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "selector": {"type": "string", "description": "CSS selector to wait for"},
                        "timeout": {
                            "type": "integer",
                            "description": "Max wait time in ms",
                            "default": 30000,
                        },
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["selector"],
                },
            },
            # ============ 截图 ============
            {
                "name": "browser_screenshot",
                "description": "Take a screenshot of the current page or a specific element.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "path": {"type": "string", "description": "Output file path"},
                        "fullPage": {
                            "type": "boolean",
                            "description": "Capture full page",
                            "default": False,
                        },
                        "ref": {
                            "type": "string",
                            "description": "Element ref to screenshot (optional)",
                        },
                        "annotate": {
                            "type": "boolean",
                            "description": "Draw red boxes around interactive elements",
                            "default": False,
                        },
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["path"],
                },
            },
            # ============ 执行 JS ============
            {
                "name": "browser_evaluate",
                "description": (
                    "Execute JavaScript code in the browser. Use for advanced operations "
                    "not covered by other tools. Returns the evaluation result."
                ),
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "script": {"type": "string", "description": "JavaScript code to execute"},
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                    "required": ["script"],
                },
            },
            # ============ 后退/前进/刷新 ============
            {
                "name": "browser_go_back",
                "description": "Go back to the previous page.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                },
            },
            {
                "name": "browser_go_forward",
                "description": "Go forward to the next page.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                },
            },
            {
                "name": "browser_reload",
                "description": "Reload the current page.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                        "targetId": {"type": "string", "description": "Tab ID (optional)"},
                    },
                },
            },
            # ============ 标签页管理 ============
            {
                "name": "browser_tabs_list",
                "description": "List all open browser tabs.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                    },
                },
            },
            {
                "name": "browser_tabs_open",
                "description": "Open a new browser tab, optionally navigating to a URL.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "url": {
                            "type": "string",
                            "description": "URL to open (default: about:blank)",
                            "default": "about:blank",
                        },
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                    },
                },
            },
            {
                "name": "browser_tabs_close",
                "description": "Close a browser tab by its target ID.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "targetId": {"type": "string", "description": "Tab ID to close"},
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                    },
                    "required": ["targetId"],
                },
            },
            {
                "name": "browser_tabs_focus",
                "description": "Switch focus to a specific tab by its target ID.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "targetId": {"type": "string", "description": "Tab ID to focus"},
                        "browserId": {
                            "type": "string",
                            "description": "Browser instance ID (optional)",
                        },
                    },
                    "required": ["targetId"],
                },
            },
            # ============ 浏览器管理 ============
            {
                "name": "browser_start",
                "description": "Start a new browser instance.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "profile": {"type": "string", "description": "Profile name (optional)"},
                    },
                },
            },
            {
                "name": "browser_close",
                "description": "Close a browser instance.",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "browserId": {"type": "string", "description": "Browser ID to close"},
                    },
                    "required": ["browserId"],
                },
            },
            {
                "name": "browser_status",
                "description": "Get current browser server status.",
                "inputSchema": {
                    "type": "object",
                    "properties": {},
                },
            },
        ]

    async def handle_tool_call(self, name: str, arguments: dict[str, Any]) -> str:
        """处理工具调用 - 增强版路由"""
        try:
            # 导航
            if name == "browser_navigate":
                resp = await self.client.post(f"{self.base_url}/api/v1/navigate", json=arguments)
                return self._format_response(resp)

            # 导航+快照（复合）
            elif name == "browser_navigate_and_snapshot":
                resp = await self.client.post(
                    f"{self.base_url}/api/v1/navigate_and_snapshot", json=arguments
                )
                data = resp.json()
                # 精简返回：只返回 LLM 需要的字段
                return json.dumps(
                    {
                        "url": data.get("url"),
                        "title": data.get("title"),
                        "summary": data.get("summary"),
                        "interactive": data.get("interactive"),
                    },
                    ensure_ascii=False,
                )

            # 快照
            elif name == "browser_snapshot":
                resp = await self.client.post(f"{self.base_url}/api/v1/snapshot", json=arguments)
                data = resp.json()
                return json.dumps(
                    {
                        "url": data.get("url"),
                        "title": data.get("title"),
                        "summary": data.get("summary"),
                        "interactive": data.get("interactive"),
                    },
                    ensure_ascii=False,
                )

            # 点击
            elif name == "browser_click":
                resp = await self.client.post(f"{self.base_url}/api/v1/act/click", json=arguments)
                data = resp.json()
                # 返回操作结果 + 新快照（精简版）
                result = {"clicked": data.get("clicked")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 输入
            elif name == "browser_type":
                resp = await self.client.post(f"{self.base_url}/api/v1/act/type", json=arguments)
                data = resp.json()
                result = {"typed": data.get("typed")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 填充
            elif name == "browser_fill":
                resp = await self.client.post(f"{self.base_url}/api/v1/act/fill", json=arguments)
                data = resp.json()
                result = {"filled": data.get("filled")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 批量填表
            elif name == "browser_fill_form":
                resp = await self.client.post(
                    f"{self.base_url}/api/v1/act/fill_form", json=arguments
                )
                data = resp.json()
                result = {
                    "fieldsFilled": data.get("fieldsFilled"),
                    "submitted": data.get("submitted"),
                }
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "interactive": data["snapshot"].get("interactive"),
                        "summary": data["snapshot"].get("summary"),
                    }
                return json.dumps(result, ensure_ascii=False)

            elif name == "browser_scroll":
                resp = await self.client.post(f"{self.base_url}/api/v1/scroll", json=arguments)
                data = resp.json()
                result = {"scrolled": True}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 文件上传
            elif name == "browser_upload_file":
                resp = await self.client.post(f"{self.base_url}/api/v1/act/upload", json=arguments)
                data = resp.json()
                result = {"uploaded": data.get("uploaded")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 弹窗管理
            elif name == "browser_handle_dialog":
                resp = await self.client.post(
                    f"{self.base_url}/api/v1/dialog/handle", json=arguments
                )
                return self._format_response(resp)

            elif name == "browser_dialog_history":
                resp = await self.client.get(f"{self.base_url}/api/v1/dialog/history")
                return self._format_response(resp)

            # 下载查询
            elif name == "browser_get_downloads":
                resp = await self.client.get(f"{self.base_url}/api/v1/act/downloads")
                return self._format_response(resp)

            elif name == "browser_hover":
                resp = await self.client.post(f"{self.base_url}/api/v1/act/hover", json=arguments)
                data = resp.json()
                result = {"hovered": data.get("hovered")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 选择
            elif name == "browser_select":
                resp = await self.client.post(f"{self.base_url}/api/v1/act/select", json=arguments)
                data = resp.json()
                result = {"selected": data.get("selected")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            elif name == "browser_press":
                resp = await self.client.post(f"{self.base_url}/api/v1/act/press", json=arguments)
                data = resp.json()
                result = {"key": data.get("key")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 获取文本
            elif name == "browser_get_text":
                resp = await self.client.post(f"{self.base_url}/api/v1/get_text", json=arguments)
                data = resp.json()
                return json.dumps(
                    {"text": data.get("data", {}).get("text", "")}, ensure_ascii=False
                )

            # 获取属性
            elif name == "browser_get_attribute":
                resp = await self.client.post(
                    f"{self.base_url}/api/v1/get_attribute", json=arguments
                )
                data = resp.json()
                return json.dumps(
                    {
                        "attribute": data.get("data", {}).get("attribute"),
                        "value": data.get("data", {}).get("value"),
                    },
                    ensure_ascii=False,
                )

            # 等待元素
            elif name == "browser_wait_for_element":
                resp = await self.client.post(
                    f"{self.base_url}/api/v1/act/wait_for_element", json=arguments
                )
                data = resp.json()
                return json.dumps({"found": data.get("found", False)}, ensure_ascii=False)

            # 截图
            elif name == "browser_screenshot":
                resp = await self.client.post(f"{self.base_url}/api/v1/screenshot", json=arguments)
                return self._format_response(resp)

            elif name == "browser_evaluate":
                resp = await self.client.post(
                    f"{self.base_url}/api/v1/act/evaluate", json=arguments
                )
                data = resp.json()
                result = {"result": data.get("data", {}).get("result")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 后退
            elif name == "browser_go_back":
                resp = await self.client.post(f"{self.base_url}/api/v1/go-back", params=arguments)
                data = resp.json()
                result = {"url": data.get("url")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 前进
            elif name == "browser_go_forward":
                resp = await self.client.post(
                    f"{self.base_url}/api/v1/go-forward", params=arguments
                )
                data = resp.json()
                result = {"url": data.get("url")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 刷新
            elif name == "browser_reload":
                resp = await self.client.post(f"{self.base_url}/api/v1/reload", params=arguments)
                data = resp.json()
                result = {"url": data.get("url")}
                if "snapshot" in data:
                    result["snapshot"] = {
                        "url": data["snapshot"].get("url"),
                        "title": data["snapshot"].get("title"),
                        "summary": data["snapshot"].get("summary"),
                        "interactive": data["snapshot"].get("interactive"),
                    }
                return json.dumps(result, ensure_ascii=False)

            # 标签页操作
            elif name == "browser_tabs_list":
                params = {}
                if "browserId" in arguments:
                    params["browserId"] = arguments["browserId"]
                resp = await self.client.get(f"{self.base_url}/api/v1/tabs", params=params)
                return self._format_response(resp)

            elif name == "browser_tabs_open":
                resp = await self.client.post(f"{self.base_url}/api/v1/tabs/open", json=arguments)
                return self._format_response(resp)

            elif name == "browser_tabs_close":
                resp = await self.client.post(f"{self.base_url}/api/v1/tabs/close", json=arguments)
                return self._format_response(resp)

            elif name == "browser_tabs_focus":
                resp = await self.client.post(f"{self.base_url}/api/v1/tabs/focus", json=arguments)
                return self._format_response(resp)

            # 浏览器管理
            elif name == "browser_start":
                params = {}
                if "profile" in arguments:
                    params["profile"] = arguments["profile"]
                resp = await self.client.post(f"{self.base_url}/start", params=params)
                return self._format_response(resp)

            elif name == "browser_close":
                browser_id = arguments.get("browserId", "")
                resp = await self.client.post(f"{self.base_url}/browsers/{browser_id}/close")
                return self._format_response(resp)

            elif name == "browser_status":
                resp = await self.client.get(f"{self.base_url}/")
                return self._format_response(resp)

            else:
                return json.dumps({"error": f"Unknown tool: {name}"})

        except httpx.HTTPError as e:
            return json.dumps({"error": f"HTTP error: {str(e)}"})
        except Exception as e:
            return json.dumps({"error": str(e)})

    def _format_response(self, resp: httpx.Response) -> str:
        """格式化 HTTP 响应"""
        try:
            data = resp.json()
            # 移除冗余字段，精简返回
            for key in ["ok", "success", "elements", "text"]:
                data.pop(key, None)
            # 去掉空值
            data = {k: v for k, v in data.items() if v is not None}
            return json.dumps(data, ensure_ascii=False)
        except Exception:
            return resp.text

    # ============ MCP STDIO 协议实现 ============

    async def run(self):
        """运行 MCP 服务器（STDIO 模式）"""
        reader = asyncio.StreamReader()
        protocol = asyncio.StreamReaderProtocol(reader)
        await asyncio.get_event_loop().connect_read_pipe(lambda: protocol, sys.stdin.buffer)

        writer_transport, writer_protocol = await asyncio.get_event_loop().connect_write_pipe(
            asyncio.streams.FlowControlMixin, sys.stdout.buffer
        )
        writer = asyncio.StreamWriter(
            writer_transport, writer_protocol, reader, asyncio.get_event_loop()
        )

        while True:
            try:
                line = await reader.readline()
                if not line:
                    break

                line = line.decode("utf-8").strip()
                if not line:
                    continue

                request = json.loads(line)
                response = await self._handle_request(request)

                if response:
                    response_line = json.dumps(response, ensure_ascii=False) + "\n"
                    writer.write(response_line.encode("utf-8"))
                    await writer.drain()

            except json.JSONDecodeError:
                continue
            except Exception as e:
                logger.error(f"MCP error: {e}")
                continue

    async def _handle_request(self, request: dict[str, Any]) -> dict[str, Any] | None:
        """处理 MCP 请求"""
        method = request.get("method", "")
        req_id = request.get("id")

        if method == "initialize":
            return {
                "jsonrpc": "2.0",
                "id": req_id,
                "result": {
                    "protocolVersion": "2024-11-05",
                    "capabilities": {
                        "tools": {"listChanged": False},
                    },
                    "serverInfo": {
                        "name": "qianji",
                        "version": "2.0.0",
                    },
                    "instructions": (
                        "Browser automation tool for web navigation and interaction. "
                        "Key tools: browser_navigate_and_snapshot (navigate + get page state), "
                        "browser_click (click element), browser_type (type text), "
                        "browser_fill (clear and fill input). "
                        "Always use browser_navigate_and_snapshot first when visiting a new URL."
                    ),
                },
            }

        elif method == "tools/list":
            return {
                "jsonrpc": "2.0",
                "id": req_id,
                "result": {
                    "tools": self.tools,
                },
            }

        elif method == "tools/call":
            params = request.get("params", {})
            name = params.get("name", "")
            arguments = params.get("arguments", {})

            result = await self.handle_tool_call(name, arguments)

            return {
                "jsonrpc": "2.0",
                "id": req_id,
                "result": {
                    "content": [
                        {
                            "type": "text",
                            "text": result,
                        }
                    ],
                    "isError": "error" in result.lower() if isinstance(result, str) else False,
                },
            }

        elif method == "notifications/initialized":
            return None  # 不需要回复

        elif method == "ping":
            return {
                "jsonrpc": "2.0",
                "id": req_id,
                "result": {},
            }

        return {
            "jsonrpc": "2.0",
            "id": req_id,
            "error": {
                "code": -32601,
                "message": f"Method not found: {method}",
            },
        }


def main():
    """MCP 服务器入口点"""
    server = QianjiMCPServer()
    asyncio.run(server.run())


def sync_main():
    """同步入口点（用于 pyproject.toml scripts）"""
    main()


if __name__ == "__main__":
    main()



"""Pytest configuration and fixtures."""

import asyncio
from collections.abc import AsyncGenerator, Generator
from unittest.mock import AsyncMock, MagicMock, patch

import pytest


@pytest.fixture(scope="session")
def event_loop() -> Generator[asyncio.AbstractEventLoop, None, None]:
    """Create an instance of the default event loop for the test session."""
    loop = asyncio.get_event_loop_policy().new_event_loop()
    yield loop
    loop.close()


@pytest.fixture
async def mock_browser_manager() -> AsyncGenerator[MagicMock, None]:
    """Mock browser manager fixture."""
    with patch("qianji.core.browser_manager.BrowserManager") as mock:
        instance = MagicMock()
        instance.start = AsyncMock(return_value=True)
        instance.stop = AsyncMock(return_value=True)
        instance.is_running = True
        mock.return_value = instance
        yield instance


@pytest.fixture
async def mock_playwright() -> AsyncGenerator[MagicMock, None]:
    """Mock playwright fixture."""
    with patch("qianji.core.pw_client.async_playwright") as mock:
        pw = MagicMock()
        browser = MagicMock()
        context = MagicMock()
        page = MagicMock()

        browser.new_context = AsyncMock(return_value=context)
        context.new_page = AsyncMock(return_value=page)
        pw.chromium.launch = AsyncMock(return_value=browser)

        mock.return_value.__aenter__ = AsyncMock(return_value=pw)
        mock.return_value.__aexit__ = AsyncMock(return_value=None)

        yield pw


@pytest.fixture
def sample_config() -> dict:
    """Sample configuration fixture."""
    return {
        "port": 18796,
        "headless": True,
        "no_sandbox": False,
        "idle_timeout": 3600,
        "max_lifetime": 3600,
    }



"""
API 响应模型 - 对应 OpenClaw 的响应格式
"""

from typing import Any

from pydantic import BaseModel


class ErrorResponse(BaseModel):
    """错误响应"""

    error: str
    success: bool = False


class TabInfo(BaseModel):
    """标签页信息"""

    targetId: str
    url: str
    title: str
    active: bool = False


class StatusResponse(BaseModel):
    """状态响应"""

    enabled: bool
    profile: str
    running: bool
    cdpReady: bool
    cdpHttp: bool
    pid: int | None = None
    cdpPort: int | None = None
    cdpUrl: str | None = None
    chosenBrowser: str | None = None
    detectedBrowser: str | None = None
    detectedExecutablePath: str | None = None
    detectError: str | None = None
    userDataDir: str | None = None
    color: str = "#FF4500"
    headless: bool = True
    noSandbox: bool = False
    executablePath: str | None = None
    attachOnly: bool = False


class NavigateResponse(BaseModel):
    """导航响应"""

    ok: bool = True
    success: bool = True
    targetId: str
    url: str
    title: str | None = None
    snapshot: dict[str, Any] | None = None


class SnapshotElementData(BaseModel):
    """快照元素数据"""

    ref: str
    type: str
    name: str | None = None
    role: str | None = None
    text: str | None = None
    selector: str | None = None
    ariaLabel: str | None = None
    placeholder: str | None = None
    value: str | None = None
    checked: bool | None = None
    disabled: bool | None = None
    bbox: dict[str, float] | None = None


class SnapshotResponse(BaseModel):
    """快照响应"""

    ok: bool = True
    success: bool = True
    targetId: str
    url: str
    title: str
    text: str
    summary: str
    interactive: str
    domHash: str | None = None
    totalElements: int | None = None
    elements: dict[str, SnapshotElementData]
    viewport: dict[str, int]
    scroll: dict[str, int]
    metadata: dict[str, Any] | None = None


class ClickResponse(BaseModel):
    """点击响应"""

    ok: bool = True
    success: bool = True
    targetId: str
    url: str


class TypeResponse(BaseModel):
    """输入响应"""

    ok: bool = True
    success: bool = True
    targetId: str


class ScreenshotResponse(BaseModel):
    """截图响应"""

    ok: bool = True
    success: bool = True
    path: str
    targetId: str
    url: str


class PDFResponse(BaseModel):
    """PDF响应"""

    ok: bool = True
    success: bool = True
    path: str
    targetId: str
    url: str


class GenericResponse(BaseModel):
    """通用响应"""

    ok: bool = True
    success: bool = True
    targetId: str | None = None
    url: str | None = None
    snapshot: dict[str, Any] | None = None
    data: dict[str, Any] | None = None



"""
智能 MCP 配置生成器
自动检测项目路径，生成可直接使用的 MCP 配置
支持防止重复启动检查
"""

import json
import os
import platform
import socket
import sys
from pathlib import Path
from typing import Any


def find_project_root() -> Path:
    """自动查找项目根目录"""
    # 从当前文件开始向上查找
    current = Path(__file__).resolve()

    # 向上查找包含 pyproject.toml 或 setup.py 的目录
    for parent in [current.parent] + list(current.parents):
        if (parent / "pyproject.toml").exists() or (parent / "setup.py").exists():
            return parent

    # 如果没找到，返回当前文件的父目录
    return current.parent.parent


def find_python_executable() -> str:
    """查找 Python 解释器路径"""
    # 优先使用当前虚拟环境的 Python
    if hasattr(sys, "real_prefix") or (
        hasattr(sys, "base_prefix") and sys.base_prefix != sys.prefix
    ):
        # 在虚拟环境中
        return sys.executable

    # 尝试查找项目中的虚拟环境
    project_root = find_project_root()

    venv_paths = [
        project_root / ".venv" / "bin" / "python",
        project_root / ".venv" / "Scripts" / "python.exe",
        project_root / "venv" / "bin" / "python",
        project_root / "venv" / "Scripts" / "python.exe",
        project_root / "env" / "bin" / "python",
        project_root / "env" / "Scripts" / "python.exe",
    ]

    for venv_python in venv_paths:
        if venv_python.exists():
            return str(venv_python)

    # 回退到系统 Python
    return sys.executable


def find_free_port(start_port: int = 18796, max_port: int = 18850) -> int:
    """查找可用端口"""
    for port in range(start_port, max_port):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            try:
                s.bind(("localhost", port))
                return port
            except OSError:
                continue
    raise RuntimeError(f"No free port found between {start_port} and {max_port}")


def is_server_running(port: int) -> bool:
    """检查服务器是否已在运行"""
    try:
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            s.settimeout(1)
            result = s.connect_ex(("localhost", port))
            return result == 0
    except:
        return False


def get_running_server_port(start_port: int = 18796, max_port: int = 18850) -> int | None:
    """查找正在运行的服务器端口"""
    for port in range(start_port, max_port):
        if is_server_running(port):
            # 验证是否是 qianji 服务器
            try:
                import urllib.request

                with urllib.request.urlopen(
                    f"http://localhost:{port}/health", timeout=2
                ) as response:
                    data = json.loads(response.read())
                    if data.get("status") == "ok":
                        return port
            except:
                continue
    return None


def generate_mcp_config(
    server_port: int | None = None,
    auto_start: bool = True,
    custom_env: dict[str, str] | None = None,
) -> dict[str, Any]:
    """
    生成 MCP 配置

    Args:
        server_port: 指定服务器端口，None 则自动查找
        auto_start: 是否自动启动服务器
        custom_env: 自定义环境变量

    Returns:
        MCP 配置字典
    """
    project_root = find_project_root()
    python_path = "nohup " + find_python_executable()

    # 确定服务器端口
    if server_port is None:
        # 先检查是否有正在运行的服务器
        running_port = get_running_server_port()
        if running_port:
            server_port = running_port
            auto_start = False  # 服务器已在运行，不需要自动启动
        else:
            server_port = find_free_port()

    # 构建环境变量
    env = {
        "PYTHONPATH": str(project_root),
        "QIANJI_CONTROL_PORT": str(server_port),
    }

    # 添加自定义环境变量
    if custom_env:
        env.update(custom_env)

    # 根据是否自动启动构建配置
    if auto_start:
        # 自动启动服务器模式
        # 使用一个包装脚本来启动服务器和 MCP
        wrapper_script = project_root / "qianji" / "mcp_wrapper.py"

        config = {
            "mcpServers": {
                "qianji": {
                    "command": python_path,
                    "args": [str(wrapper_script), "--port", str(server_port), "--mcp"],
                    "env": env,
                }
            }
        }
    else:
        # 仅 MCP 模式（连接到已有服务器）
        mcp_script = project_root / "qianji" / "mcp.py"

        config = {
            "mcpServers": {
                "qianji": {
                    "command": python_path,
                    "args": [str(mcp_script), "--server-url", f"http://localhost:{server_port}"],
                    "env": env,
                }
            }
        }

    return config


def print_config_json(config: dict[str, Any]):
    """打印格式化的 JSON 配置"""
    print("\n" + "=" * 70)
    print("Generated MCP Configuration")
    print("=" * 70)
    print("\n复制以下配置到你的 MCP 配置文件:\n")
    print(json.dumps(config, indent=2, ensure_ascii=False))
    print("\n" + "=" * 70)


def print_config_locations():
    """打印配置文件位置"""
    system = platform.system()

    print("\n配置文件位置:")
    print("-" * 50)

    if system == "Darwin":  # macOS
        print("Claude Desktop: ~/Library/Application Support/Claude/claude_desktop_config.json")
        print("Cursor: ~/Library/Application Support/Cursor/mcp.json")
    elif system == "Windows":
        print("Claude Desktop: %APPDATA%/Claude/claude_desktop_config.json")
        print("Cursor: %APPDATA%/Cursor/mcp.json")
    else:  # Linux
        print("Claude Desktop: ~/.config/Claude/claude_desktop_config.json")
        print("Cursor: ~/.config/Cursor/mcp.json")
        print(
            "Cline: ~/.config/Code/User/globalStorage/saoudrizwan.claude-dev/settings/cline_mcp_settings.json"
        )

    print("-" * 50)


def write_config_to_file(config: dict[str, Any], filepath: Path):
    """写入配置文件"""
    filepath = Path(filepath)
    filepath.parent.mkdir(parents=True, exist_ok=True)

    # 如果文件存在，合并配置
    if filepath.exists():
        try:
            with open(filepath) as f:
                existing = json.load(f)
        except json.JSONDecodeError:
            existing = {}

        if "mcpServers" not in existing:
            existing["mcpServers"] = {}

        existing["mcpServers"]["qianji"] = config["mcpServers"]["qianji"]

        with open(filepath, "w") as f:
            json.dump(existing, f, indent=2, ensure_ascii=False)
    else:
        with open(filepath, "w") as f:
            json.dump(config, f, indent=2, ensure_ascii=False)

    print(f"\n✓ Configuration written to: {filepath}")


def main():
    """主入口"""
    import argparse

    parser = argparse.ArgumentParser(
        description="Generate MCP configuration for qianji",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  # 生成配置（自动检测）
  python -m qianji.mcp_config

  # 指定端口
  python -m qianji.mcp_config --port 18797

  # 不自动启动服务器（连接到已有服务器）
  python -m qianji.mcp_config --no-auto-start

  # 写入 Claude Desktop 配置
  python -m qianji.mcp_config --write --target claude

  # 添加自定义环境变量
  python -m qianji.mcp_config --env KEY=value --env ANOTHER=val
        """,
    )

    parser.add_argument("--port", type=int, default=None, help="Server port (default: auto-detect)")
    parser.add_argument(
        "--no-auto-start",
        action="store_true",
        help="Don't auto-start server, connect to existing one",
    )
    parser.add_argument("--write", action="store_true", help="Write configuration to file")
    parser.add_argument(
        "--target",
        choices=["claude", "cursor", "cline"],
        default="claude",
        help="Target application for --write (default: claude)",
    )
    parser.add_argument(
        "--env", action="append", default=[], help="Additional environment variables (KEY=value)"
    )
    parser.add_argument(
        "--output", type=Path, default=None, help="Output file path (default: print to stdout)"
    )

    args = parser.parse_args()

    # 解析环境变量
    custom_env = {}
    for env_var in args.env:
        if "=" in env_var:
            key, value = env_var.split("=", 1)
            custom_env[key] = value

    # 检查是否有正在运行的服务器
    running_port = get_running_server_port()
    if running_port and not args.port:
        print(f"✓ Found running qianji server on port {running_port}")
        print("  Will connect to existing server\n")
        server_port = running_port
        auto_start = False
    else:
        server_port = args.port
        auto_start = not args.no_auto_start

    # 生成配置
    config = generate_mcp_config(
        server_port=server_port,
        auto_start=auto_start,
        custom_env=custom_env if custom_env else None,
    )

    # 输出配置
    if args.write:
        system = platform.system()
        if args.target == "claude":
            if system == "Darwin":
                config_path = (
                    Path.home()
                    / "Library"
                    / "Application Support"
                    / "Claude"
                    / "claude_desktop_config.json"
                )
            elif system == "Windows":
                config_path = (
                    Path(os.environ.get("APPDATA", "")) / "Claude" / "claude_desktop_config.json"
                )
            else:
                config_path = Path.home() / ".config" / "Claude" / "claude_desktop_config.json"
        elif args.target == "cursor":
            if system == "Darwin":
                config_path = (
                    Path.home() / "Library" / "Application Support" / "Cursor" / "mcp.json"
                )
            elif system == "Windows":
                config_path = Path(os.environ.get("APPDATA", "")) / "Cursor" / "mcp.json"
            else:
                config_path = Path.home() / ".config" / "Cursor" / "mcp.json"
        else:  # cline
            config_path = (
                Path.home()
                / ".config"
                / "Code"
                / "User"
                / "globalStorage"
                / "saoudrizwan.claude-dev"
                / "settings"
                / "cline_mcp_settings.json"
            )

        write_config_to_file(config, config_path)
    elif args.output:
        write_config_to_file(config, args.output)
    else:
        print_config_json(config)
        print_config_locations()

    # 打印使用说明
    print("\n使用说明:")
    print("-" * 50)
    if auto_start:
        print("1. 将此配置添加到 MCP 客户端")
        print("2. 启动 MCP 客户端时会自动启动 qianji 服务器")
        print("3. 服务器将在后台运行，多个 MCP 连接共享同一实例")
    else:
        print("1. 确保 qianji 服务器已在运行")
        print(f"   qianji-server --port {server_port}")
        print("2. 将此配置添加到 MCP 客户端")
        print("3. MCP 客户端将连接到现有服务器")
    print("-" * 50)


if __name__ == "__main__":
    main()



"""
弹窗/对话框路由 - V3
处理 alert/confirm/prompt 弹窗
"""

import logging

from fastapi import APIRouter, HTTPException
from pydantic import BaseModel

from ..core import get_pw_client

logger = logging.getLogger("qianji.routes.dialog")
router = APIRouter()


class DialogModeRequest(BaseModel):
    """设置弹窗处理策略"""

    mode: str = "accept"  # accept / dismiss / custom
    customText: str | None = None


class DialogRespondRequest(BaseModel):
    """手动响应当前弹窗"""

    action: str = "accept"
    text: str | None = None


@router.post("/dialog/handle")
async def set_dialog_mode(request: DialogModeRequest):
    """设置弹窗处理策略

    - accept: 自动接受所有弹窗（默认）
    - dismiss: 自动关闭所有弹窗
    - custom: 用自定义文本接受 prompt 弹窗
    """
    pw = get_pw_client()

    if request.mode not in ("accept", "dismiss", "custom"):
        raise HTTPException(status_code=400, detail=f"Invalid mode: {request.mode}")

    pw.set_dialog_mode(request.mode, request.customText)

    return {
        "mode": request.mode,
        "customText": request.customText,
    }


@router.get("/dialog/history")
async def get_dialog_history():
    """获取弹窗历史记录"""
    pw = get_pw_client()
    history = pw.get_dialog_history()
    return {
        "dialogs": history,
        "count": len(history),
    }



"""Logging configuration for qianji."""

import logging
import sys
from typing import Any

import structlog


def configure_logging(
    level: str = "INFO",
    json_format: bool = False,
    log_file: str | None = None,
) -> None:
    """Configure structured logging for qianji.

    Args:
        level: Logging level (DEBUG, INFO, WARNING, ERROR)
        json_format: Whether to use JSON format
        log_file: Optional log file path
    """
    # Configure standard library logging
    logging.basicConfig(
        format="%(message)s",
        stream=sys.stdout,
        level=getattr(logging, level.upper()),
    )

    # Configure structlog
    shared_processors: list = [
        structlog.stdlib.filter_by_level,
        structlog.stdlib.add_logger_name,
        structlog.stdlib.add_log_level,
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.processors.TimeStamper(fmt="iso"),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
    ]

    if json_format:
        # JSON format for production
        processors = shared_processors + [structlog.processors.JSONRenderer()]
    else:
        # Console format for development
        processors = shared_processors + [structlog.dev.ConsoleRenderer(colors=sys.stdout.isatty())]

    structlog.configure(
        processors=processors,
        context_class=dict,
        logger_factory=structlog.stdlib.LoggerFactory(),
        wrapper_class=structlog.stdlib.BoundLogger,
        cache_logger_on_first_use=True,
    )

    # Add file handler if specified
    if log_file:
        file_handler = logging.FileHandler(log_file)
        file_handler.setLevel(getattr(logging, level.upper()))
        root_logger = logging.getLogger()
        root_logger.addHandler(file_handler)


def get_logger(name: str) -> structlog.stdlib.BoundLogger:
    """Get a structured logger instance.

    Args:
        name: Logger name

    Returns:
        Structured logger instance
    """
    return structlog.get_logger(name)


class RequestLogger:
    """Logger for HTTP requests."""

    def __init__(self, logger: structlog.stdlib.BoundLogger):
        self.logger = logger

    def log_request(
        self,
        method: str,
        path: str,
        status_code: int,
        duration_ms: float,
        **extra: Any,
    ) -> None:
        """Log an HTTP request.

        Args:
            method: HTTP method
            path: Request path
            status_code: Response status code
            duration_ms: Request duration in milliseconds
            **extra: Additional fields to log
        """
        log_data: dict[str, Any] = {
            "event": "http_request",
            "method": method,
            "path": path,
            "status_code": status_code,
            "duration_ms": round(duration_ms, 2),
        }
        log_data.update(extra)

        if status_code >= 500:
            self.logger.error(**log_data)
        elif status_code >= 400:
            self.logger.warning(**log_data)
        else:
            self.logger.info(**log_data)



"""
qianji v2 - 完全复刻 OpenClaw Browser
基于 Playwright 的 Python 实现

100% API 兼容 OpenClaw browser 系统
"""

from .core.browser_manager import BrowserManager
from .models.config import BrowserConfig, ProfileConfig
from .models.snapshot import ElementRef, Snapshot
from .server import BrowserServer

__version__ = "2.0.0"
__all__ = [
    "BrowserServer",
    "BrowserManager",
    "BrowserConfig",
    "ProfileConfig",
    "Snapshot",
    "ElementRef",
]



"""
MCP 安装脚本
配置 Claude Desktop 和 Cursor 使用 qianji MCP
"""

import json
import os
import platform
import sys
from pathlib import Path


def get_claude_config_path() -> Path:
    """获取 Claude Desktop 配置路径"""
    system = platform.system()
    if system == "Darwin":  # macOS
        return (
            Path.home()
            / "Library"
            / "Application Support"
            / "Claude"
            / "claude_desktop_config.json"
        )
    elif system == "Windows":
        return Path(os.environ.get("APPDATA", "")) / "Claude" / "claude_desktop_config.json"
    else:  # Linux
        return Path.home() / ".config" / "Claude" / "claude_desktop_config.json"


def get_cursor_config_path() -> Path:
    """获取 Cursor 配置路径"""
    system = platform.system()
    if system == "Darwin":  # macOS
        return Path.home() / "Library" / "Application Support" / "Cursor" / "mcp.json"
    elif system == "Windows":
        return Path(os.environ.get("APPDATA", "")) / "Cursor" / "mcp.json"
    else:  # Linux
        return Path.home() / ".config" / "Cursor" / "mcp.json"


def get_mcp_server_config(
    server_url: str = "http://localhost:18796",
    python_path: str | None = None,
    auto_start: bool = True,
    idle_timeout: int | None = None,
    max_lifetime: int | None = None,
) -> dict:
    """生成 MCP 服务器配置

    Args:
        server_url: qianji 服务器 URL
        python_path: Python 解释器路径（默认自动检测）
        auto_start: 是否自动启动服务器（默认 True）
        idle_timeout: 浏览器空闲超时（秒）
        max_lifetime: 浏览器最大生命周期（秒）
    """
    # 获取 Python 解释器路径
    if python_path is None:
        python_path = sys.executable

    project_root = Path(__file__).parent.parent.absolute()
    wrapper_path = project_root / "qianji" / "mcp_wrapper.py"

    # 构建 args
    args = [
        str(wrapper_path),
        "--port",
        server_url.split(":")[-1],
    ]

    # 如果不自动启动，添加 --no-auto-start
    if not auto_start:
        args.append("--no-auto-start")

    args.append("--mcp")

    # 构建 env
    env = {"PYTHONPATH": str(project_root), "QIANJI_CONTROL_PORT": server_url.split(":")[-1]}

    # 添加生命周期配置
    if idle_timeout is not None:
        env["QIANJI_IDLE_TIMEOUT"] = str(idle_timeout)
    if max_lifetime is not None:
        env["QIANJI_MAX_LIFETIME"] = str(max_lifetime)

    return {"command": python_path, "args": args, "env": env}


def setup_claude_mcp(server_url: str = "http://localhost:18796"):
    """配置 Claude Desktop MCP"""
    config_path = get_claude_config_path()
    config_path.parent.mkdir(parents=True, exist_ok=True)

    # 读取现有配置
    config = {}
    if config_path.exists():
        try:
            with open(config_path) as f:
                config = json.load(f)
        except json.JSONDecodeError:
            print(f"Warning: Could not parse existing config at {config_path}")
            config = {}

    # 确保 mcpServers 存在
    if "mcpServers" not in config:
        config["mcpServers"] = {}

    # 添加 qianji MCP 服务器
    config["mcpServers"]["qianji"] = get_mcp_server_config(server_url)

    # 写入配置
    with open(config_path, "w") as f:
        json.dump(config, f, indent=2)

    print(f"✓ Claude Desktop MCP configured at: {config_path}")
    print(f"  Server URL: {server_url}")


def setup_cursor_mcp(server_url: str = "http://localhost:18796"):
    """配置 Cursor MCP"""
    config_path = get_cursor_config_path()
    config_path.parent.mkdir(parents=True, exist_ok=True)

    # 读取现有配置
    config = {}
    if config_path.exists():
        try:
            with open(config_path) as f:
                config = json.load(f)
        except json.JSONDecodeError:
            print(f"Warning: Could not parse existing config at {config_path}")
            config = {}

    # 确保 mcpServers 存在
    if "mcpServers" not in config:
        config["mcpServers"] = {}

    # 添加 qianji MCP 服务器
    config["mcpServers"]["qianji"] = get_mcp_server_config(server_url)

    # 写入配置
    with open(config_path, "w") as f:
        json.dump(config, f, indent=2)

    print(f"✓ Cursor MCP configured at: {config_path}")
    print(f"  Server URL: {server_url}")


def print_manual_config(server_url: str = "http://localhost:18796", python_path: str | None = None):
    """打印手动配置说明"""
    if python_path is None:
        python_path = sys.executable

    print("\n" + "=" * 60)
    print("Manual MCP Configuration")
    print("=" * 60)
    print("\nAdd the following to your MCP configuration:\n")
    project_root = Path(__file__).parent.parent.absolute()
    wrapper_path = project_root / "qianji" / "mcp_wrapper.py"

    config = {
        "mcpServers": {
            "qianji": {
                "command": python_path,
                "args": [str(wrapper_path), "--port", server_url.split(":")[-1], "--mcp"],
                "env": {
                    "PYTHONPATH": str(project_root),
                    "QIANJI_CONTROL_PORT": server_url.split(":")[-1],
                },
            }
        }
    }

    print(json.dumps(config, indent=2))
    print("\n" + "=" * 60)
    print("\nConfiguration Notes:")
    print("  - The wrapper will auto-start the qianji server if not running")
    print("  - Server will be accessible at " + server_url)
    print("  - Environment variables:")
    print("    - QIANJI_IDLE_TIMEOUT: Browser idle timeout (seconds)")
    print("    - QIANJI_MAX_LIFETIME: Browser max lifetime (seconds)")
    print("  - To disable auto-start, add '--no-auto-start' to args")
    print("=" * 60)


def main():
    """MCP 安装脚本入口"""
    import argparse

    parser = argparse.ArgumentParser(
        description="Setup qianji MCP for Claude Desktop and Cursor",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  # Configure both Claude and Cursor with auto-start
  python -m qianji.mcp_setup

  # Configure with custom Python path
  python -m qianji.mcp_setup --python /root/miniconda3/envs/py312/bin/python

  # Configure without auto-start (manual server management)
  python -m qianji.mcp_setup --no-auto-start

  # Print manual configuration
  python -m qianji.mcp_setup --manual
        """,
    )
    parser.add_argument(
        "--server-url",
        default="http://localhost:18796",
        help="qianji server URL (default: http://localhost:18796)",
    )
    parser.add_argument(
        "--python",
        dest="python_path",
        default=None,
        help="Python interpreter path (default: auto-detect)",
    )
    parser.add_argument("--claude", action="store_true", help="Configure Claude Desktop only")
    parser.add_argument("--cursor", action="store_true", help="Configure Cursor only")
    parser.add_argument("--manual", action="store_true", help="Print manual configuration only")
    parser.add_argument(
        "--no-auto-start",
        action="store_true",
        help="Don't auto-start server (manual server management)",
    )
    parser.add_argument(
        "--idle-timeout",
        type=int,
        default=None,
        help="Browser idle timeout in seconds (default: 3600)",
    )
    parser.add_argument(
        "--max-lifetime",
        type=int,
        default=None,
        help="Browser max lifetime in seconds (default: 3600)",
    )

    args = parser.parse_args()

    if args.manual:
        print_manual_config(args.server_url, args.python_path)
        return

    # 如果没有指定，默认配置所有
    configure_all = not (args.claude or args.cursor)

    print("Setting up qianji MCP...")
    print(f"Server URL: {args.server_url}")
    print(f"Python: {args.python_path or 'auto-detect'}")
    print(f"Auto-start: {not args.no_auto_start}")
    if args.idle_timeout:
        print(f"Idle timeout: {args.idle_timeout}s")
    if args.max_lifetime:
        print(f"Max lifetime: {args.max_lifetime}s")
    print()

    # 更新全局配置函数参数
    global get_mcp_server_config
    original_get_config = get_mcp_server_config

    def get_mcp_server_config_with_args(server_url):
        return original_get_config(
            server_url=server_url,
            python_path=args.python_path,
            auto_start=not args.no_auto_start,
            idle_timeout=args.idle_timeout,
            max_lifetime=args.max_lifetime,
        )

    # 替换函数
    import qianji.mcp_setup

    qianji.mcp_setup.get_mcp_server_config = get_mcp_server_config_with_args

    if configure_all or args.claude:
        try:
            setup_claude_mcp(args.server_url)
        except Exception as e:
            print(f"✗ Failed to configure Claude Desktop: {e}")

    if configure_all or args.cursor:
        try:
            setup_cursor_mcp(args.server_url)
        except Exception as e:
            print(f"✗ Failed to configure Cursor: {e}")

    print("\n" + "=" * 60)
    print("Setup complete!")
    print("=" * 60)
    print("\nNext steps:")
    if args.no_auto_start:
        print("1. Start qianji server manually:")
        print(f"   python -m qianji.server --port {args.server_url.split(':')[-1]}")
        print("2. Restart Claude Desktop or Cursor")
    else:
        print("1. Restart Claude Desktop or Cursor")
        print("2. The MCP wrapper will auto-start the server when needed")
    print("3. The browser tools will be available in your AI assistant")
    print("\nAvailable tools:")
    print("  - browser_navigate: Navigate to a URL")
    print("  - browser_navigate_and_snapshot: Navigate and get snapshot")
    print("  - browser_snapshot: Get page snapshot with interactive elements")
    print("  - browser_click: Click an element")
    print("  - browser_fill: Fill a form field")
    print("  - browser_type: Type text")
    print("  - browser_upload_file: Upload file(s)")
    print("  - browser_handle_dialog: Handle dialogs")
    print("  - browser_screenshot: Take a screenshot")
    print("  - browser_evaluate: Execute JavaScript")
    print("  - browser_go_back: Go back")
    print("  - browser_go_forward: Go forward")
    print("  - browser_reload: Reload page")


def sync_main():
    """同步入口"""
    main()


if __name__ == "__main__":
    main()



"""API 路由"""

from fastapi import APIRouter

from .agent_act import router as agent_act_router
from .agent_debug import router as agent_debug_router
from .agent_dialog import router as agent_dialog_router
from .agent_snapshot import router as agent_snapshot_router
from .agent_storage import router as agent_storage_router
from .basic import router as basic_router
from .tabs import router as tabs_router


def create_api_router() -> APIRouter:
    """创建 API 路由 (用于向后兼容)"""
    api_router = APIRouter()

    # 基础路由
    api_router.include_router(basic_router, tags=["basic"])

    # Agent 路由
    api_router.include_router(agent_snapshot_router, prefix="", tags=["snapshot"])
    api_router.include_router(agent_act_router, prefix="/act", tags=["act"])
    api_router.include_router(agent_storage_router, prefix="/storage", tags=["storage"])
    api_router.include_router(agent_debug_router, prefix="/debug", tags=["debug"])
    api_router.include_router(agent_dialog_router, prefix="", tags=["dialog"])
    api_router.include_router(tabs_router, prefix="/tabs", tags=["tabs"])

    return api_router



"""
浏览器管理器 - 对应 OpenClaw browser/chrome.js + profiles-service.js
"""

import os
from typing import Any

from playwright.async_api import Browser, BrowserContext, async_playwright

from ..models.config import BrowserConfig, ProfileConfig
from .tab_manager import TabManager


def get_default_user_data_dir() -> str:
    """获取默认的用户数据目录"""
    # 使用用户主目录下的 .qianji/user_data 文件夹
    base_dir = os.path.expanduser("~/.qianji")
    user_data_dir = os.path.join(base_dir, "user_data")
    os.makedirs(user_data_dir, exist_ok=True)
    return user_data_dir


# 反检测脚本 - 注入到每个页面以隐藏自动化特征
STEALTH_SCRIPTS = """
(() => {
    // 覆盖 navigator.webdriver
    Object.defineProperty(navigator, 'webdriver', {
        get: () => undefined,
        configurable: true
    });
    
    // 删除 chrome 自动化属性
    delete navigator.__proto__.webdriver;
    
    // 注入 window.chrome 对象
    if (!window.chrome) {
        window.chrome = {};
    }
    
    // 添加 chrome 运行时属性
    window.chrome.runtime = {
        id: undefined,
        OnInstalledReason: {
            CHROME_UPDATE: "chrome_update",
            INSTALL: "install",
            SHARED_MODULE_UPDATE: "shared_module_update",
            UPDATE: "update"
        },
        OnRestartRequiredReason: {
            APP_UPDATE: "app_update",
            OS_UPDATE: "os_update",
            PERIODIC: "periodic"
        },
        PlatformArch: {
            ARM: "arm",
            ARM64: "arm64",
            MIPS: "mips",
            MIPS64: "mips64",
            X86_32: "x86-32",
            X86_64: "x86-64"
        },
        PlatformNaclArch: {
            ARM: "arm",
            MIPS: "mips",
            MIPS64: "mips64",
            MIPS64_EL: "mips64el",
            MIPS_EL: "mipsel",
            X86_32: "x86-32",
            X86_64: "x86-64"
        },
        PlatformOs: {
            ANDROID: "android",
            CROS: "cros",
            LINUX: "linux",
            MAC: "mac",
            OPENBSD: "openbsd",
            WIN: "win"
        },
        RequestUpdateCheckStatus: {
            NO_UPDATE: "no_update",
            THROTTLED: "throttled",
            UPDATE_AVAILABLE: "update_available"
        }
    };
    
    // 添加 chrome app 属性
    window.chrome.app = {
        isInstalled: false,
        InstallState: {
            DISABLED: "disabled",
            INSTALLED: "installed",
            NOT_INSTALLED: "not_installed"
        },
        RunningState: {
            CANNOT_RUN: "cannot_run",
            READY_TO_RUN: "ready_to_run",
            RUNNING: "running"
        }
    };
    
    // 添加 chrome csi 属性
    window.chrome.csi = function() {
        return {
            onloadT: Date.now(),
            pageT: Date.now() - performance.timing.navigationStart,
            startE: performance.timing.navigationStart,
            type: "nav"
        };
    };
    
    // 添加 chrome loadTimes 属性
    window.chrome.loadTimes = function() {
        return {
            commitLoadTime: performance.timing.domContentLoadedEventStart / 1000,
            connectionInfo: "h2",
            finishDocumentLoadTime: performance.timing.domContentLoadedEventEnd / 1000,
            finishLoadTime: performance.timing.loadEventEnd / 1000,
            firstPaintAfterLoadTime: 0,
            firstPaintTime: performance.timing.domContentLoadedEventStart / 1000,
            navigationType: "Other",
            npnNegotiatedProtocol: "h2",
            requestTime: performance.timing.requestStart / 1000,
            startLoadTime: performance.timing.navigationStart / 1000,
            wasAlternateProtocolAvailable: false,
            wasFetchedViaSpdy: true,
            wasNpnNegotiated: true
        };
    };
    
    // 覆盖 permissions.query 以绕过权限检测
    const originalQuery = window.navigator.permissions.query;
    window.navigator.permissions.query = (parameters) => (
        parameters.name === 'notifications' ?
            Promise.resolve({ state: Notification.permission }) :
            originalQuery(parameters)
    );
    
    // 模拟真实的插件列表
    const plugins = [
        {
            name: "Chrome PDF Plugin",
            filename: "internal-pdf-viewer",
            description: "Portable Document Format",
            version: "undefined",
            length: 1,
            item: function() { return this; }
        },
        {
            name: "Chrome PDF Viewer",
            filename: "mhjfbmdgcfjbbpaeojofohoefgiehjai",
            description: "Portable Document Format",
            version: "undefined",
            length: 1,
            item: function() { return this; }
        },
        {
            name: "Native Client",
            filename: "internal-nacl-plugin",
            description: "Native Client module",
            version: "undefined",
            length: 2,
            item: function() { return this; }
        }
    ];
    
    // 覆盖 navigator.plugins
    Object.defineProperty(navigator, 'plugins', {
        get: () => {
            const pluginArray = plugins;
            pluginArray.length = plugins.length;
            pluginArray.item = (index) => plugins[index] || null;
            pluginArray.namedItem = (name) => plugins.find(p => p.name === name) || null;
            return pluginArray;
        },
        configurable: true
    });
    
    // 覆盖 navigator.mimeTypes
    Object.defineProperty(navigator, 'mimeTypes', {
        get: () => {
            const mimeTypes = [
                { type: "application/pdf", suffixes: "pdf", description: "Portable Document Format", enabledPlugin: plugins[0] },
                { type: "application/x-google-chrome-pdf", suffixes: "pdf", description: "Portable Document Format", enabledPlugin: plugins[1] },
                { type: "application/x-nacl", suffixes: "", description: "Native Client executable", enabledPlugin: plugins[2] },
                { type: "application/x-pnacl", suffixes: "", description: "Portable Native Client executable", enabledPlugin: plugins[2] }
            ];
            const mimeTypeArray = mimeTypes;
            mimeTypeArray.length = mimeTypes.length;
            mimeTypeArray.item = (index) => mimeTypes[index] || null;
            mimeTypeArray.namedItem = (name) => mimeTypes.find(m => m.type === name) || null;
            return mimeTypeArray;
        },
        configurable: true
    });
    
    // 覆盖 Notification.permission
    Object.defineProperty(Notification, 'permission', {
        get: () => "default",
        configurable: true
    });
    
    // 修复 Canvas 指纹 - 添加轻微噪声
    const originalToDataURL = HTMLCanvasElement.prototype.toDataURL;
    HTMLCanvasElement.prototype.toDataURL = function(type) {
        if (this.width > 0 && this.height > 0) {
            const ctx = this.getContext('2d');
            if (ctx) {
                const imageData = ctx.getImageData(0, 0, this.width, this.height);
                const data = imageData.data;
                // 添加微小的随机噪声（几乎不可见但改变哈希值）
                for (let i = 0; i < data.length; i += 4) {
                    if (Math.random() < 0.001) {
                        data[i] = Math.max(0, Math.min(255, data[i] + (Math.random() > 0.5 ? 1 : -1)));
                    }
                }
                ctx.putImageData(imageData, 0, 0);
            }
        }
        return originalToDataURL.apply(this, arguments);
    };
    
    // 覆盖 getContext 以支持 WebGL
    const originalGetContext = HTMLCanvasElement.prototype.getContext;
    HTMLCanvasElement.prototype.getContext = function(contextType) {
        if (contextType === 'webgl' || contextType === 'experimental-webgl' || contextType === 'webgl2') {
            const gl = originalGetContext.apply(this, arguments);
            if (gl) {
                // 覆盖 WebGL 参数以隐藏自动化特征
                const originalGetParameter = gl.getParameter;
                gl.getParameter = function(parameter) {
                    // 隐藏 UNMASKED_VENDOR_WEBGL 和 UNMASKED_RENDERER_WEBGL
                    if (parameter === 0x9245) { // UNMASKED_VENDOR_WEBGL
                        return "Intel Inc.";
                    }
                    if (parameter === 0x9246) { // UNMASKED_RENDERER_WEBGL
                        return "Intel Iris OpenGL Engine";
                    }
                    return originalGetParameter.apply(this, arguments);
                };
            }
            return gl;
        }
        return originalGetContext.apply(this, arguments);
    };
    
    // 覆盖 WebGL2RenderingContext
    if (window.WebGL2RenderingContext) {
        const originalWebGL2GetParameter = window.WebGL2RenderingContext.prototype.getParameter;
        window.WebGL2RenderingContext.prototype.getParameter = function(parameter) {
            if (parameter === 0x9245) {
                return "Intel Inc.";
            }
            if (parameter === 0x9246) {
                return "Intel Iris OpenGL Engine";
            }
            return originalWebGL2GetParameter.apply(this, arguments);
        };
    }
    
    // 覆盖 navigator.languages
    Object.defineProperty(navigator, 'languages', {
        get: () => ["zh-CN", "zh", "en-US", "en"],
        configurable: true
    });
    
    // 覆盖 navigator.deviceMemory
    Object.defineProperty(navigator, 'deviceMemory', {
        get: () => 8,
        configurable: true
    });
    
    // 覆盖 navigator.hardwareConcurrency
    Object.defineProperty(navigator, 'hardwareConcurrency', {
        get: () => 8,
        configurable: true
    });
    
    // 覆盖 screen 属性
    Object.defineProperty(screen, 'colorDepth', {
        get: () => 24,
        configurable: true
    });
    
    Object.defineProperty(screen, 'pixelDepth', {
        get: () => 24,
        configurable: true
    });
    
    // 修复 iframe 的 contentWindow
    const originalCreateElement = document.createElement;
    document.createElement = function(tagName) {
        const element = originalCreateElement.apply(this, arguments);
        if (tagName.toLowerCase() === 'iframe') {
            try {
                const originalContentWindow = element.contentWindow;
                Object.defineProperty(element, 'contentWindow', {
                    get: () => {
                        const win = originalContentWindow;
                        if (win) {
                            win.navigator.webdriver = undefined;
                        }
                        return win;
                    },
                    configurable: true
                });
            } catch (e) {}
        }
        return element;
    };
    
    // 覆盖 console.debug 以防止某些检测
    const originalConsoleDebug = console.debug;
    console.debug = function() {
        // 过滤掉自动化相关的调试信息
        const args = Array.from(arguments);
        if (args.some(arg => typeof arg === 'string' && arg.includes('automation'))) {
            return;
        }
        return originalConsoleDebug.apply(this, arguments);
    };
    
    // 添加假的 plugins 刷新
    Object.defineProperty(navigator, 'plugins', {
        get: function() {
            const plugins = [
                {name: "Chrome PDF Plugin", filename: "internal-pdf-viewer", description: "Portable Document Format", version: undefined, length: 1},
                {name: "Chrome PDF Viewer", filename: "mhjfbmdgcfjbbpaeojofohoefgiehjai", description: "Portable Document Format", version: undefined, length: 1},
                {name: "Native Client", filename: "internal-nacl-plugin", description: "Native Client module", version: undefined, length: 2}
            ];
            plugins.length = 3;
            plugins.refresh = function() {};
            plugins.item = function(index) { return this[index] || null; };
            plugins.namedItem = function(name) {
                for (let i = 0; i < this.length; i++) {
                    if (this[i].name === name) return this[i];
                }
                return null;
            };
            return plugins;
        },
        configurable: true
    });
})();
"""


class BrowserManager:
    """浏览器管理器 - 管理 Playwright 浏览器实例"""

    def __init__(self, config: BrowserConfig):
        self.config = config
        self.playwright = None
        self.browser: Browser | None = None
        self.context: BrowserContext | None = None
        self.tab_manager = TabManager()
        self._profile: ProfileConfig | None = None
        self._user_data_dir: str | None = None
        self._running = False

    @property
    def is_running(self) -> bool:
        """浏览器是否运行中"""
        return self._running and self.browser is not None

    def _get_user_data_dir(self, profile: ProfileConfig) -> str:
        """获取用户数据目录"""
        if profile.user_data_dir:
            # 使用配置文件指定的目录
            user_data_dir = os.path.expanduser(profile.user_data_dir)
        else:
            # 使用默认目录，按配置文件分目录
            base_dir = get_default_user_data_dir()
            user_data_dir = os.path.join(base_dir, profile.name)

        # 确保目录存在
        os.makedirs(user_data_dir, exist_ok=True)
        return user_data_dir

    async def start(
        self, profile_name: str | None = None, user_data_dir: str | None = None
    ) -> bool:
        """启动浏览器

        Args:
            profile_name: 配置文件名称
            user_data_dir: 用户数据目录（可选，如果不提供则使用配置文件中的设置）
        """
        if self.is_running:
            return True

        profile = self.config.get_profile(profile_name)
        self._profile = profile

        # 获取用户数据目录
        if user_data_dir:
            # 使用传入的用户数据目录（用于多浏览器实例）
            self._user_data_dir = user_data_dir
        else:
            # 使用配置文件中的用户数据目录
            self._user_data_dir = self._get_user_data_dir(profile)

        # 启动 Playwright
        self.playwright = await async_playwright().start()

        # 浏览器启动参数
        browser_args = []

        # 检测是否为 root 用户，如果是则自动添加 no-sandbox 参数
        if os.getuid() == 0 or profile.no_sandbox:
            browser_args.extend(
                [
                    "--no-sandbox",
                    "--disable-setuid-sandbox",
                    "--disable-gpu",
                    "--disable-dev-shm-usage",
                ]
            )

        # 反检测参数 - 隐藏自动化特征，避免被识别为机器人
        browser_args.extend(
            [
                "--disable-blink-features=AutomationControlled",
                "--disable-web-security",
                "--disable-features=IsolateOrigins,site-per-process,AutomationControlled",
                "--disable-software-rasterizer",
                "--disable-logging",
                "--log-level=3",
                "--remote-debugging-port=0",
                "--disable-background-networking",
                "--disable-default-apps",
                "--disable-sync",
                "--disable-translate",
                "--metrics-recording-only",
                "--mute-audio",
                "--no-first-run",
                "--safebrowsing-disable-auto-update",
                # 启用 WebGL 支持
                "--enable-webgl",
                "--enable-features=WebGL,WebGL2",
                # 禁用自动化提示
                "--disable-infobars",
                "--disable-popup-blocking",
                # 窗口大小和位置
                "--window-size=1920,1080",
                "--window-position=0,0",
                # 禁用自动化扩展
                "--disable-extensions-except=",
                "--disable-component-extensions-with-background-pages",
                # 用户代理相关
                "--force-device-scale-factor=1",
                "--device-scale-factor=1",
                # 内存和性能
                "--memory-model=low",
                "--max_old_space_size=4096",
            ]
        )

        # 添加自定义参数
        browser_args.extend(profile.args)

        # 确定可执行路径
        executable_path = profile.executable_path or self.config.executable_path

        # 启动浏览器
        try:
            # 统一使用 Windows 平台的 User-Agent 和配置
            # 确保 User-Agent 和 Platform 一致
            user_agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36"

            # 额外的上下文选项 - 用于反检测
            context_options = {
                "user_data_dir": self._user_data_dir,
                "headless": profile.headless,
                "args": browser_args,
                "user_agent": user_agent,
                "viewport": {"width": 1920, "height": 1080},
                "screen": {"width": 1920, "height": 1080},
                "device_scale_factor": 1,
                "is_mobile": False,
                "has_touch": False,
                "locale": "zh-CN",
                "timezone_id": "Asia/Shanghai",
                "geolocation": {"latitude": 39.9042, "longitude": 116.4074},  # 北京
                "permissions": [],
                "color_scheme": "light",
                "reduced_motion": "no-preference",
                "forced_colors": "none",
            }

            # 添加可执行路径（如果指定）
            if executable_path:
                context_options["executable_path"] = executable_path

            # 启动浏览器
            self.browser = await self.playwright.chromium.launch_persistent_context(
                **context_options
            )
            # launch_persistent_context 返回的是 context，不是 browser
            self.context = self.browser

            self._running = True

            # 为所有页面添加反检测脚本
            self.context.on("page", self._on_page_created)

            # 创建第一个标签页（如果使用 persistent context，可能已经有一个页面了）
            pages = self.context.pages
            if pages:
                page = pages[0]
                await self._inject_stealth_scripts(page)
            else:
                page = await self.context.new_page()
            await self.tab_manager.create_tab(page)

            return True

        except Exception as e:
            await self.stop()
            raise RuntimeError(f"Failed to start browser: {e}")

    async def stop(self) -> bool:
        """停止浏览器"""
        if not self.is_running:
            return False

        # 关闭所有标签页
        await self.tab_manager.close_all()

        # 关闭上下文（对于 persistent context，这就是 browser）
        if self.context:
            try:
                await self.context.close()
            except:
                pass
            self.context = None

        self.browser = None

        # 停止 Playwright
        if self.playwright:
            try:
                await self.playwright.stop()
            except:
                pass
            self.playwright = None

        self._running = False
        self._profile = None
        # 注意：不清理 _user_data_dir，以保留 cookie 和 session

        return True

    async def reset(self) -> bool:
        """重置浏览器（关闭并重新启动）"""
        profile_name = self._profile.name if self._profile else None
        await self.stop()
        return await self.start(profile_name)

    async def _on_page_created(self, page):
        """当新页面创建时注入反检测脚本"""
        await self._inject_stealth_scripts(page)

    async def _inject_stealth_scripts(self, page):
        """注入反检测脚本到页面"""
        try:
            # 在页面加载前注入脚本
            await page.add_init_script(STEALTH_SCRIPTS)
        except Exception:
            # 忽略注入失败的错误
            pass

    def get_status(self) -> dict[str, Any]:
        """获取浏览器状态"""
        return {
            "running": self.is_running,
            "profile": self._profile.name if self._profile else None,
            "tabs": len(self.tab_manager.tabs),
            "activeTab": self.tab_manager.active_tab_id,
            "userDataDir": self._user_data_dir,
        }



"""
Core 包 - 共享单例
"""

from .browser_manager import BrowserManager
from .browser_registry import BrowserRegistry
from .pw_client import PlaywrightClient, SnapshotCache, snapshot_cache
from .tab_manager import TabManager

# 全局单例 PlaywrightClient（所有路由共享）
_pw_client: PlaywrightClient = PlaywrightClient()


def get_pw_client() -> PlaywrightClient:
    """获取全局 PlaywrightClient 单例"""
    return _pw_client


def get_snapshot_cache() -> SnapshotCache:
    """获取全局快照缓存"""
    return snapshot_cache



"""
命令行接口 - 提供 CLI 工具
支持多浏览器架构
"""

import argparse
import asyncio
import json
import os
import subprocess
import sys
import time
from typing import Any

import httpx


def is_root_user() -> bool:
    """检测是否为 root 用户"""
    try:
        return os.getuid() == 0
    except AttributeError:
        # Windows 系统没有 getuid
        return False


def is_server_running(base_url: str) -> bool:
    """检测服务器是否正在运行"""
    try:
        import urllib.request

        urllib.request.urlopen(f"{base_url}/", timeout=2)
        return True
    except:
        return False


def start_server_in_background(port: int = 18796) -> bool:
    """在后台启动服务器（使用 nohup）"""
    try:
        # 获取 Python 解释器路径
        python_cmd = sys.executable

        # 构建启动命令
        cmd = [
            "nohup",
            python_cmd,
            "-m",
            "qianji.server",
            "--port",
            str(port),
            "--no-sandbox" if is_root_user() else "",
        ]
        # 过滤空字符串
        cmd = [c for c in cmd if c]

        # 启动服务器（后台运行）
        log_file = "/tmp/qianji_server.log"
        with open(log_file, "w") as f:
            subprocess.Popen(
                cmd, stdout=f, stderr=subprocess.STDOUT, stdin=subprocess.DEVNULL, close_fds=True
            )

        # 等待服务器启动
        base_url = f"http://localhost:{port}"
        for i in range(30):  # 最多等待 30 秒
            if is_server_running(base_url):
                return True
            time.sleep(1)

        return False
    except Exception as e:
        print(f"启动服务器失败: {e}")
        return False


async def ensure_server(base_url: str) -> bool:
    """确保服务器正在运行"""
    if is_server_running(base_url):
        return True

    # 从 URL 中提取端口
    try:
        port = int(base_url.split(":")[-1].split("/")[0])
    except:
        port = 18796

    print(f"⏳ 服务器未运行，正在启动（端口 {port}）...")

    if start_server_in_background(port):
        print(f"✓ 服务器已启动: {base_url}")
        return True
    else:
        print("✗ 服务器启动失败")
        return False


def output_result(data: Any, format_type: str = "text", prefix: str = ""):
    """统一输出结果

    Args:
        data: 要输出的数据
        format_type: 输出格式，"text" 或 "json"
        prefix: text 格式时的前缀
    """
    if format_type == "json":
        if isinstance(data, dict) or isinstance(data, list):
            print(json.dumps(data, indent=2, ensure_ascii=False))
        else:
            print(json.dumps({"result": data}, indent=2, ensure_ascii=False))
    else:
        if isinstance(data, dict):
            for key, value in data.items():
                if isinstance(value, dict):
                    print(f"{prefix}{key}:")
                    output_result(value, format_type, prefix + "  ")
                elif isinstance(value, list):
                    print(f"{prefix}{key}:")
                    for i, item in enumerate(value):
                        if isinstance(item, dict):
                            print(f"{prefix}  [{i}]:")
                            output_result(item, format_type, prefix + "    ")
                        else:
                            print(f"{prefix}  - {item}")
                else:
                    print(f"{prefix}{key}: {value}")
        elif isinstance(data, list):
            for i, item in enumerate(data):
                if isinstance(item, dict):
                    print(f"{prefix}[{i}]:")
                    output_result(item, format_type, prefix + "  ")
                else:
                    print(f"{prefix}- {item}")
        else:
            print(f"{prefix}{data}")


class QianjiClient:
    """qianji HTTP 客户端 - 支持多浏览器"""

    def __init__(self, base_url: str = "http://localhost:18796"):
        self.base_url = base_url
        self.client = httpx.AsyncClient(timeout=30.0)

    async def close(self):
        await self.client.aclose()

    async def status(self) -> dict:
        """获取状态"""
        resp = await self.client.get(f"{self.base_url}/")
        resp.raise_for_status()
        return resp.json()

    async def start(
        self,
        profile: str | None = None,
        headless: bool | None = None,
        no_sandbox: bool | None = None,
        args: list | None = None,
    ) -> dict:
        """启动浏览器"""
        params = {}
        if profile:
            params["profile"] = profile
        if headless is not None:
            params["headless"] = headless
        if no_sandbox is not None:
            params["no_sandbox"] = no_sandbox
        if args:
            params["args"] = args
        resp = await self.client.post(f"{self.base_url}/start", params=params)
        resp.raise_for_status()
        return resp.json()

    async def stop(self, browser_id: str | None = None) -> dict:
        """停止浏览器"""
        params = {"browser_id": browser_id} if browser_id else None
        resp = await self.client.post(f"{self.base_url}/stop", params=params)
        resp.raise_for_status()
        return resp.json()

    async def list_browsers(self) -> dict:
        """列出所有浏览器"""
        resp = await self.client.get(f"{self.base_url}/browsers")
        resp.raise_for_status()
        return resp.json()

    async def create_browser(
        self,
        profile: str | None = None,
        browser_id: str | None = None,
        headless: bool | None = None,
        no_sandbox: bool | None = None,
        args: list | None = None,
    ) -> dict:
        """创建新浏览器"""
        params = {}
        if profile:
            params["profile"] = profile
        if browser_id:
            params["browser_id"] = browser_id
        if headless is not None:
            params["headless"] = headless
        if no_sandbox is not None:
            params["no_sandbox"] = no_sandbox
        if args:
            params["args"] = args
        resp = await self.client.post(f"{self.base_url}/browsers/create", params=params)
        resp.raise_for_status()
        return resp.json()

    async def close_browser(self, browser_id: str) -> dict:
        """关闭指定浏览器"""
        resp = await self.client.post(
            f"{self.base_url}/browsers/close", params={"browser_id": browser_id}
        )
        resp.raise_for_status()
        return resp.json()

    async def list_tabs(self, browser_id: str | None = None) -> dict:
        """列出所有标签页"""
        params = {"browserId": browser_id} if browser_id else None
        resp = await self.client.get(f"{self.base_url}/api/v1/tabs", params=params)
        resp.raise_for_status()
        return resp.json()

    async def open_tab(self, url: str = "about:blank", browser_id: str | None = None) -> dict:
        """打开新标签页"""
        data = {"url": url}
        if browser_id:
            data["browserId"] = browser_id
        resp = await self.client.post(f"{self.base_url}/api/v1/tabs/open", json=data)
        resp.raise_for_status()
        return resp.json()

    async def close_tab(self, target_id: str, browser_id: str | None = None) -> dict:
        """关闭标签页"""
        data = {"targetId": target_id}
        if browser_id:
            data["browserId"] = browser_id
        resp = await self.client.post(f"{self.base_url}/api/v1/tabs/close", json=data)
        resp.raise_for_status()
        return resp.json()

    async def navigate(
        self, url: str, browser_id: str | None = None, target_id: str | None = None
    ) -> dict:
        """导航"""
        data = {"url": url}
        if browser_id:
            data["browserId"] = browser_id
        if target_id:
            data["targetId"] = target_id
        resp = await self.client.post(f"{self.base_url}/api/v1/navigate", json=data)
        resp.raise_for_status()
        return resp.json()

    async def snapshot(self, browser_id: str | None = None, target_id: str | None = None) -> dict:
        """获取快照"""
        data = {}
        if browser_id:
            data["browserId"] = browser_id
        if target_id:
            data["targetId"] = target_id
        resp = await self.client.post(f"{self.base_url}/api/v1/snapshot", json=data)
        resp.raise_for_status()
        return resp.json()

    async def click(
        self, ref: str, browser_id: str | None = None, target_id: str | None = None
    ) -> dict:
        """点击"""
        data = {"ref": ref}
        if browser_id:
            data["browserId"] = browser_id
        if target_id:
            data["targetId"] = target_id
        resp = await self.client.post(f"{self.base_url}/api/v1/act/click", json=data)
        resp.raise_for_status()
        return resp.json()

    async def type(
        self,
        ref: str,
        text: str,
        browser_id: str | None = None,
        target_id: str | None = None,
        submit: bool = False,
    ) -> dict:
        """输入"""
        data = {"ref": ref, "text": text, "submit": submit}
        if browser_id:
            data["browserId"] = browser_id
        if target_id:
            data["targetId"] = target_id
        resp = await self.client.post(f"{self.base_url}/api/v1/act/type", json=data)
        resp.raise_for_status()
        return resp.json()

    async def screenshot(
        self,
        path: str,
        browser_id: str | None = None,
        target_id: str | None = None,
        full_page: bool = False,
    ) -> dict:
        """截图"""
        data = {"path": path, "fullPage": full_page}
        if browser_id:
            data["browserId"] = browser_id
        if target_id:
            data["targetId"] = target_id
        resp = await self.client.post(f"{self.base_url}/api/v1/screenshot", json=data)
        resp.raise_for_status()
        return resp.json()

    async def fill(
        self, ref: str, text: str, browser_id: str | None = None, target_id: str | None = None
    ) -> dict:
        """填充表单"""
        data = {"ref": ref, "text": text}
        if browser_id:
            data["browserId"] = browser_id
        if target_id:
            data["targetId"] = target_id
        resp = await self.client.post(f"{self.base_url}/api/v1/act/fill", json=data)
        resp.raise_for_status()
        return resp.json()

    async def evaluate(
        self, script: str, browser_id: str | None = None, target_id: str | None = None
    ) -> dict:
        """执行 JavaScript"""
        data = {"script": script}
        if browser_id:
            data["browserId"] = browser_id
        if target_id:
            data["targetId"] = target_id
        resp = await self.client.post(f"{self.base_url}/api/v1/act/evaluate", json=data)
        resp.raise_for_status()
        return resp.json()


async def cmd_status(client: QianjiClient, args):
    """状态命令"""
    result = await client.status()
    if args.format == "json":
        output_result(result, "json")
    else:
        print("Server Status:")
        print(f"  Enabled: {result.get('enabled', False)}")
        print(f"  Running: {result.get('running', False)}")
        print(f"  CDP Ready: {result.get('cdpReady', False)}")
        print(f"  Profile: {result.get('profile', 'default')}")
        print(f"  Browser: {result.get('chosenBrowser', 'unknown')}")
        print(f"  Executable: {result.get('executablePath', 'unknown')}")


async def cmd_start(client: QianjiClient, args):
    """启动浏览器命令"""
    # 自动检测 root 用户，默认启用 no-sandbox
    no_sandbox = args.no_sandbox
    if is_root_user() and not args.sandbox:
        no_sandbox = True
        if args.format == "text":
            print("ℹ️ 检测到 root 用户，自动启用 --no-sandbox（可使用 --sandbox 强制禁用）")

    # 构建额外参数列表
    extra_args = []
    if args.window_size:
        extra_args.append(f"--window-size={args.window_size}")
    if args.user_agent:
        extra_args.append(f"--user-agent={args.user_agent}")
    if args.proxy:
        extra_args.append(f"--proxy-server={args.proxy}")
    if args.disable_dev_shm:
        extra_args.append("--disable-dev-shm-usage")
    if args.disable_gpu:
        extra_args.append("--disable-gpu")
    if args.disable_extensions:
        extra_args.append("--disable-extensions")

    result = await client.start(
        profile=args.profile,
        headless=args.headless,
        no_sandbox=no_sandbox,
        args=extra_args if extra_args else None,
    )

    if args.format == "json":
        output_result(result, "json")
    else:
        browser_id = result.get("browserId", "unknown")
        status = result.get("status", "unknown")
        print(f"✓ Browser {browser_id} is {status}")
        print(f"  Profile: {result.get('profile', 'default')}")
        if status == "starting":
            print("  ⏳ 浏览器正在后台启动，请稍等几秒后使用 'qianji browsers' 查看状态")


async def cmd_stop(client: QianjiClient, args):
    """停止浏览器命令"""
    result = await client.stop(args.browser)
    if args.format == "json":
        output_result(result, "json")
    else:
        print("✓ Browser stopped")
        output_result(result.get("data", {}), "text", "  ")


async def cmd_browsers(client: QianjiClient, args):
    """列出所有浏览器命令"""
    result = await client.list_browsers()
    data = result.get("data", {})

    if args.format == "json":
        output_result(result, "json")
    else:
        print(f"Browsers: {data.get('totalBrowsers', 0)}")
        print(f"Active: {data.get('runningBrowsers', 0)}")
        print(f"Total Tabs: {data.get('totalTabs', 0)}")
        for info in data.get("browsers", []):
            bid = info.get("browserId", "unknown")
            print(f"\n  {bid}:")
            print(f"    Profile: {info.get('profileName', 'default')}")
            print(f"    Running: {info.get('running', False)}")
            print(f"    Tabs: {info.get('tabs', 0)}")
            print(f"    User Data: {info.get('userDataDir', 'N/A')}")


async def cmd_new_browser(client: QianjiClient, args):
    """创建新浏览器命令"""
    # 自动检测 root 用户，默认启用 no-sandbox
    no_sandbox = args.no_sandbox
    if is_root_user() and not args.sandbox:
        no_sandbox = True
        if args.format == "text":
            print("ℹ️ 检测到 root 用户，自动启用 --no-sandbox（可使用 --sandbox 强制禁用）")

    # 构建额外参数列表
    extra_args = []
    if args.window_size:
        extra_args.append(f"--window-size={args.window_size}")
    if args.user_agent:
        extra_args.append(f"--user-agent={args.user_agent}")
    if args.proxy:
        extra_args.append(f"--proxy-server={args.proxy}")
    if args.disable_dev_shm:
        extra_args.append("--disable-dev-shm-usage")
    if args.disable_gpu:
        extra_args.append("--disable-gpu")
    if args.disable_extensions:
        extra_args.append("--disable-extensions")

    result = await client.create_browser(
        profile=args.profile,
        browser_id=args.id,
        headless=args.headless,
        no_sandbox=no_sandbox,
        args=extra_args if extra_args else None,
    )

    if args.format == "json":
        output_result(result, "json")
    else:
        browser_id = result.get("browserId", "unknown")
        status = result.get("status", "unknown")
        print(f"✓ Browser {browser_id} is {status}")
        if status == "starting":
            print("  ⏳ 浏览器正在后台启动，请稍等几秒后使用 'qianji browsers' 查看状态")


async def cmd_close_browser(client: QianjiClient, args):
    """关闭指定浏览器命令"""
    result = await client.close_browser(args.browser_id)
    if args.format == "json":
        output_result(result, "json")
    else:
        print(f"✓ Browser closed: {args.browser_id}")


async def cmd_tabs(client: QianjiClient, args):
    """列出标签页命令"""
    result = await client.list_tabs(args.browser)
    data = result.get("data", {})

    if args.format == "json":
        output_result(result, "json")
    else:
        if args.browser:
            print(f"Tabs for {args.browser}:")
            for tab in data.get("tabs", []):
                print(f"  {tab.get('targetId')}: {tab.get('title', 'N/A')}")
                print(f"    URL: {tab.get('url', 'N/A')}")
        else:
            for browser_id, tabs in data.get("tabs", {}).items():
                print(f"\n{browser_id}:")
                for tab in tabs:
                    print(f"  {tab.get('targetId')}: {tab.get('title', 'N/A')}")


async def cmd_navigate(client: QianjiClient, args):
    """导航命令"""
    result = await client.navigate(args.url, args.browser, args.target)

    if args.format == "json":
        output_result(result, "json")
    else:
        print(f"✓ Navigated to: {result.get('url')}")
        print(f"  Title: {result.get('title')}")


async def cmd_snapshot(client: QianjiClient, args):
    """快照命令"""
    result = await client.snapshot(args.browser, args.target)

    if args.format == "json":
        output_result(result, "json")
    else:
        print(f"✓ Snapshot of: {result.get('url')}")
        print(f"  Title: {result.get('title')}")
        print("\n  Interactive elements:")
        print(result.get("interactive", "None"))


async def cmd_click(client: QianjiClient, args):
    """点击命令"""
    result = await client.click(args.ref, args.browser, args.target)
    if args.format == "json":
        output_result(result, "json")
    else:
        print(f"✓ Clicked: {args.ref}")


async def cmd_type(client: QianjiClient, args):
    """输入命令"""
    result = await client.type(args.ref, args.text, args.browser, args.target, args.submit)
    if args.format == "json":
        output_result(result, "json")
    else:
        print(f"✓ Typed into: {args.ref}")


async def cmd_screenshot(client: QianjiClient, args):
    """截图命令"""
    result = await client.screenshot(args.output, args.browser, args.target, args.full_page)
    if args.format == "json":
        output_result(result, "json")
    else:
        print(f"✓ Screenshot saved: {result.get('path')}")


async def cmd_fill(client: QianjiClient, args):
    """填充命令"""
    result = await client.fill(args.ref, args.text, args.browser, args.target)
    if args.format == "json":
        output_result(result, "json")
    else:
        print(f"✓ Filled: {args.ref}")


async def cmd_evaluate(client: QianjiClient, args):
    """执行 JS 命令"""
    result = await client.evaluate(args.script, args.browser, args.target)
    if args.format == "json":
        output_result(result, "json")
    else:
        print("✓ Result:")
        print(json.dumps(result.get("data", {}).get("result"), indent=2, ensure_ascii=False))


def add_format_argument(parser):
    """为 parser 添加 format 参数"""
    parser.add_argument(
        "--format", choices=["text", "json"], default="text", help="Output format (default: text)"
    )


async def main():
    """主函数"""
    parser = argparse.ArgumentParser(
        description="qianji CLI - Universal Browser Automation (Multi-Browser Support)",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  qianji status                    # 查看状态
  qianji browsers                  # 列出所有浏览器
  qianji new-browser --id chrome1  # 创建新浏览器
  qianji start                     # 启动默认浏览器
  qianji navigate https://example.com --browser chrome1
  qianji snapshot                  # 获取页面快照
  qianji click e1 --browser chrome1 --target tab1
  qianji type e2 "hello world" --browser chrome1
  qianji screenshot output.png --browser chrome1

Multi-Browser Options:
  --browser BROWSER_ID             # 指定浏览器实例
  --target TARGET_ID               # 指定标签页
  --format {text,json}             # 输出格式
        """,
    )

    parser.add_argument("--server", default="http://localhost:18796", help="Server URL")

    subparsers = parser.add_subparsers(dest="command", help="Commands")

    # status
    status_parser = subparsers.add_parser("status", help="Get server status")
    add_format_argument(status_parser)

    # browsers - 列出所有浏览器
    browsers_parser = subparsers.add_parser("browsers", help="List all browser instances")
    add_format_argument(browsers_parser)

    # new-browser - 创建新浏览器
    new_browser_parser = subparsers.add_parser("new-browser", help="Create new browser instance")
    new_browser_parser.add_argument("--profile", help="Profile name")
    new_browser_parser.add_argument("--id", help="Browser ID (optional)")
    new_browser_parser.add_argument("--headless", action="store_true", help="Run in headless mode")
    new_browser_parser.add_argument(
        "--no-sandbox", action="store_true", help="Disable sandbox (for Docker/root)"
    )
    new_browser_parser.add_argument(
        "--sandbox", action="store_true", help="Force enable sandbox (override root auto-detection)"
    )
    new_browser_parser.add_argument("--window-size", help="Window size (e.g., 1920,1080)")
    new_browser_parser.add_argument("--user-agent", help="Custom User-Agent")
    new_browser_parser.add_argument("--proxy", help="Proxy server (e.g., http://proxy:8080)")
    new_browser_parser.add_argument(
        "--disable-dev-shm", action="store_true", help="Disable /dev/shm usage"
    )
    new_browser_parser.add_argument("--disable-gpu", action="store_true", help="Disable GPU")
    new_browser_parser.add_argument(
        "--disable-extensions", action="store_true", help="Disable extensions"
    )
    add_format_argument(new_browser_parser)

    # close-browser - 关闭浏览器
    close_browser_parser = subparsers.add_parser("close-browser", help="Close browser instance")
    close_browser_parser.add_argument("browser_id", help="Browser ID to close")
    add_format_argument(close_browser_parser)

    # start
    start_parser = subparsers.add_parser("start", help="Start default browser")
    start_parser.add_argument("--profile", help="Profile name")
    start_parser.add_argument("--headless", action="store_true", help="Run in headless mode")
    start_parser.add_argument(
        "--no-sandbox", action="store_true", help="Disable sandbox (for Docker/root)"
    )
    start_parser.add_argument(
        "--sandbox", action="store_true", help="Force enable sandbox (override root auto-detection)"
    )
    start_parser.add_argument("--window-size", help="Window size (e.g., 1920,1080)")
    start_parser.add_argument("--user-agent", help="Custom User-Agent")
    start_parser.add_argument("--proxy", help="Proxy server (e.g., http://proxy:8080)")
    start_parser.add_argument(
        "--disable-dev-shm", action="store_true", help="Disable /dev/shm usage"
    )
    start_parser.add_argument("--disable-gpu", action="store_true", help="Disable GPU")
    start_parser.add_argument(
        "--disable-extensions", action="store_true", help="Disable extensions"
    )
    add_format_argument(start_parser)

    # stop
    stop_parser = subparsers.add_parser("stop", help="Stop browser")
    stop_parser.add_argument("--browser", help="Browser ID to stop")
    add_format_argument(stop_parser)

    # tabs
    tabs_parser = subparsers.add_parser("tabs", help="List all tabs")
    tabs_parser.add_argument("--browser", help="Browser ID")
    add_format_argument(tabs_parser)

    # navigate
    nav_parser = subparsers.add_parser("navigate", help="Navigate to URL")
    nav_parser.add_argument("url", help="URL to navigate")
    nav_parser.add_argument("--browser", help="Browser ID")
    nav_parser.add_argument("--target", help="Target tab ID")
    add_format_argument(nav_parser)

    # snapshot
    snap_parser = subparsers.add_parser("snapshot", help="Get page snapshot")
    snap_parser.add_argument("--browser", help="Browser ID")
    snap_parser.add_argument("--target", help="Target tab ID")
    add_format_argument(snap_parser)

    # click
    click_parser = subparsers.add_parser("click", help="Click element")
    click_parser.add_argument("ref", help="Element ref (e.g., e1)")
    click_parser.add_argument("--browser", help="Browser ID")
    click_parser.add_argument("--target", help="Target tab ID")
    add_format_argument(click_parser)

    # type
    type_parser = subparsers.add_parser("type", help="Type text")
    type_parser.add_argument("ref", help="Element ref (e.g., e1)")
    type_parser.add_argument("text", help="Text to type")
    type_parser.add_argument("--browser", help="Browser ID")
    type_parser.add_argument("--target", help="Target tab ID")
    type_parser.add_argument("--submit", action="store_true", help="Submit after typing")
    add_format_argument(type_parser)

    # fill
    fill_parser = subparsers.add_parser("fill", help="Fill form field")
    fill_parser.add_argument("ref", help="Element ref (e.g., e1)")
    fill_parser.add_argument("text", help="Text to fill")
    fill_parser.add_argument("--browser", help="Browser ID")
    fill_parser.add_argument("--target", help="Target tab ID")
    add_format_argument(fill_parser)

    # screenshot
    screenshot_parser = subparsers.add_parser("screenshot", help="Take screenshot")
    screenshot_parser.add_argument("output", help="Output file path")
    screenshot_parser.add_argument("--browser", help="Browser ID")
    screenshot_parser.add_argument("--target", help="Target tab ID")
    screenshot_parser.add_argument("--full-page", action="store_true", help="Capture full page")
    add_format_argument(screenshot_parser)

    # evaluate
    eval_parser = subparsers.add_parser("evaluate", help="Execute JavaScript")
    eval_parser.add_argument("script", help="JavaScript code")
    eval_parser.add_argument("--browser", help="Browser ID")
    eval_parser.add_argument("--target", help="Target tab ID")
    add_format_argument(eval_parser)

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
        sys.exit(1)

    # 对于需要服务器的命令，自动检测并启动服务器
    server_required_commands = [
        "start",
        "new-browser",
        "stop",
        "tabs",
        "navigate",
        "snapshot",
        "click",
        "type",
        "fill",
        "screenshot",
        "evaluate",
        "browsers",
        "close-browser",
    ]

    if args.command in server_required_commands:
        if not await ensure_server(args.server):
            print(f"✗ 无法连接到服务器: {args.server}")
            print(
                f"  请手动启动服务器: python -m qianji.server --port {args.server.split(':')[-1]}"
            )
            sys.exit(1)

    client = QianjiClient(args.server)

    try:
        commands = {
            "status": cmd_status,
            "browsers": cmd_browsers,
            "new-browser": cmd_new_browser,
            "close-browser": cmd_close_browser,
            "start": cmd_start,
            "stop": cmd_stop,
            "tabs": cmd_tabs,
            "navigate": cmd_navigate,
            "snapshot": cmd_snapshot,
            "click": cmd_click,
            "type": cmd_type,
            "fill": cmd_fill,
            "screenshot": cmd_screenshot,
            "evaluate": cmd_evaluate,
        }

        cmd_func = commands.get(args.command)
        if cmd_func:
            await cmd_func(client, args)
        else:
            print(f"Unknown command: {args.command}")
            sys.exit(1)
    except Exception as e:
        print(f"Error: {e}")
        sys.exit(1)
    finally:
        await client.close()


if __name__ == "__main__":
    asyncio.run(main())


def sync_main():
    """同步入口函数 - 用于命令行调用"""
    asyncio.run(main())



"""Integration tests for HTTP API endpoints."""

from unittest.mock import AsyncMock, MagicMock, patch

import pytest
from fastapi.testclient import TestClient

from qianji.models.config import BrowserConfig
from qianji.server import BrowserServer


@pytest.mark.integration
class TestAPIEndpoints:
    """Test HTTP API endpoints."""

    @pytest.fixture
    def client(self) -> TestClient:
        """Create a test client."""
        config = BrowserConfig()
        server = BrowserServer(config)
        app = server.create_app()
        return TestClient(app)

    def test_status_endpoint(self, client: TestClient) -> None:
        """Test GET / status endpoint."""
        response = client.get("/")
        # 503 is acceptable when browser is not running
        assert response.status_code in [200, 503]

    def test_browsers_list_endpoint(self, client: TestClient) -> None:
        """Test GET /browsers endpoint."""
        response = client.get("/browsers")
        # 503 is acceptable when browser is not running
        assert response.status_code in [200, 503]

    @pytest.mark.asyncio
    async def test_start_browser_endpoint(self, client: TestClient) -> None:
        """Test POST /start endpoint."""
        with patch("qianji.routes.basic.get_browser_registry") as mock_registry:
            mock_reg = MagicMock()
            mock_reg.create_browser = AsyncMock(return_value="browser_1")
            mock_reg._browsers = {}
            mock_registry.return_value = mock_reg

            response = client.post("/start")
            assert response.status_code == 200
            data = response.json()
            assert data["ok"] is True

    def test_navigate_endpoint(self, client: TestClient) -> None:
        """Test POST /api/v1/navigate endpoint."""
        # Skip this test as it requires actual browser
        pytest.skip("Requires actual browser instance")



"""Unit tests for browser manager."""

import pytest

from qianji.core.browser_manager import BrowserManager
from qianji.models.config import BrowserConfig


@pytest.mark.unit
class TestBrowserManager:
    """Test BrowserManager class."""

    @pytest.fixture
    def config(self) -> BrowserConfig:
        """Create a test config."""
        return BrowserConfig()

    @pytest.fixture
    def manager(self, config: BrowserConfig) -> BrowserManager:
        """Create a browser manager instance."""
        return BrowserManager(config)

    def test_initialization(self, manager: BrowserManager, config: BrowserConfig) -> None:
        """Test browser manager initialization."""
        assert manager.config == config
        assert manager.is_running is False
        assert manager.browser is None
        assert manager.context is None

    @pytest.mark.asyncio
    async def test_stop_when_not_running(self, manager: BrowserManager) -> None:
        """Test stopping when browser is not running."""
        result = await manager.stop()
        assert result is False

    def test_get_status_not_running(self, manager: BrowserManager) -> None:
        """Test getting status when browser is not running."""
        status = manager.get_status()
        assert status["running"] is False
        assert status["profile"] is None
        assert status["tabs"] == 0
